--[[
    Broker_RecZone by LordFarlander
    Version 3.0.3
    $Revision: 2 $

    A conversion of Fubar_RecZone by wmrojer
]]--

--[[
Copyright (c) 2008, LordFarlander
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are met:

    * Redistributions of source code must retain the above copyright notice,
      this list of conditions and the following disclaimer.
    * Redistributions in binary form must reproduce the above copyright notice,
      this list of conditions and the following disclaimer in the documentation
      and/or other materials provided with the distribution.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR
ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
(INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
]]--

local L = LibStub( "AceLocale-3.0" ):GetLocale( "Broker_RecZone" );
local Tablet = LibStub( "Tablet-2.0" );
local Jostle = LibStub( "LibJostle-3.0" );

Broker_RecZone = LibStub( "AceAddon-3.0" ):NewAddon( "Broker_RecZone", "AceEvent-3.0", "AceConsole-3.0", "AceTimer-3.0" );

Broker_RecZone.version = "3.0." .. "3";
Broker_RecZone.date = string.sub("$Date: 2008-11-18 15:54:54 +0000 (Tue, 18 Nov 2008) $", 8, 17);

function Broker_RecZone:RegisterTablet( frame )
    if( not Tablet:IsRegistered( frame ) ) then
        Tablet:Register( frame,
            'children', function()
                            Tablet:SetTitle( self.ldbobj.label );
                            self:OnTooltipUpdate();
                        end,
            'clickable', true,
            'data', {},
            'detachedData', {},
            'point', function( frame )
                         if( frame:GetTop() > GetScreenHeight() / 2 ) then
                             local x = frame:GetCenter();
        
                             if( x < GetScreenWidth() / 3 ) then
                                 return "TOPLEFT", "BOTTOMLEFT";
                             elseif( x < GetScreenWidth() * 2 / 3 ) then
                                 return "TOP", "BOTTOM";
                             else
                                 return "TOPRIGHT", "BOTTOMRIGHT";
                             end--if
                         else
                             local x = frame:GetCenter();

                             if( x < GetScreenWidth() / 3 ) then
                                 return "BOTTOMLEFT", "TOPLEFT";
                             elseif( x < GetScreenWidth() * 2 / 3 ) then
                                 return "BOTTOM", "TOP";
                             else
                                 return "BOTTOMRIGHT", "TOPRIGHT";
                             end--if
                         end--if
                     end );
    end--if
end--Broker_RecZone:RegisterTablet( frame )

function Broker_RecZone:UpdateTooltip()
    if( self.tooltip.IsVisible() ) then
        self:OnTooltipUpdate();
    end--if
end--Broker_RecZone:UpdateTooltip()

-- Enumerations.
-- Faction
local RZ_ALLIANCE      = 1;
local RZ_HORDE         = 2;
local RZ_CONTESTED     = 3;
local RZ_SANCTUARY     = 4;
local RZ_PVP           = 5;
-- Continent
local RZ_KALIMDOR      = 1;
local RZ_EASTERN       = 2;
local RZ_OUTLAND       = 3;
local RZ_NORTHREND     = 4;
local RZ_OTHER         =255;
-- Type
local RZ_ZONE          = 1;
local RZ_CITY          = 2;
local RZ_TRANSPORT     = 3;
local RZ_NONE          = 4;
local RZ_INSTANCE      = 5;
local RZ_BATTLEGROUND  = 6;
local RZ_ARENA         = 7;
local RZ_RAID20        = 8;
local RZ_RAID40        = 9;
local RZ_RAID25        =10;
local RZ_RAID10        =11;
local RZ_RAID10_20     =12;

local RZ_MAXLEVEL      =80;


Broker_RecZone.vars = {
    current_continent = nil,
    current_zone = nil,
    playerLevel = 0,
    zoneName = "",
    subZoneName = "",
    playerFaction = 0,
    zonetext = "",
    coordtext = "",
    leveltext = "",
    playerCoordX = 0,
    playerCoordY = 0,
};

Broker_RecZone.vars.factions = {
    [RZ_ALLIANCE]     = L["Alliance"],
    [RZ_HORDE]        = L["Horde"],
    [RZ_CONTESTED]    = L["Contested"],
    [RZ_SANCTUARY]    = L["Sanctuary"],
    [RZ_PVP]          = L["PVP"],
};
Broker_RecZone.vars.types = {
    [RZ_ZONE]         = "",
    [RZ_CITY]         = "City",
    [RZ_TRANSPORT]    = "Transport",
    [RZ_NONE]         = "",
    [RZ_INSTANCE]     = "",
    [RZ_BATTLEGROUND] = L["Battleground"],
    [RZ_RAID20]       = L["Raid20"],
    [RZ_RAID40]       = L["Raid40"],
    [RZ_RAID25]       = L["Raid25"],
    [RZ_RAID10]       = L["Raid10"],
    [RZ_RAID10_20]    = L["Raid10or20"],
    [RZ_ARENA]        = L["Arena"],
};
Broker_RecZone.vars.continents = {};
do
    local function zn(zonename)
        for nr,zone in ipairs(Broker_RecZone.vars.zones) do
            if zone.name == zonename then
                return nr;
            end
        end
    end
    local function znx(zonename,x)
        for nr,zone in ipairs(Broker_RecZone.vars.zones) do
            if zone.name == zonename and zone.x and zone.x == x then
                return nr;
            end
        end
    end
    local Z = LibStub("LibBabble-Zone-3.0"):GetLookupTable()
    Broker_RecZone.vars.zones={
    
-- Eastern
        {name=Z["Stormwind City"],low=0,high=0,faction=RZ_ALLIANCE,type=RZ_CITY,continent=RZ_EASTERN},
        {name=Z["Ironforge"],low=0,high=0,faction=RZ_ALLIANCE,type=RZ_CITY,continent=RZ_EASTERN},
        {name=Z["Undercity"],low=0,high=0,faction=RZ_HORDE,type=RZ_CITY,continent=RZ_EASTERN},
        {name=Z["Dun Morogh"],low=1,high=12,faction=RZ_ALLIANCE,type=RZ_ZONE,continent=RZ_EASTERN},
        {name=Z["Elwynn Forest"],low=1,high=12,faction=RZ_ALLIANCE,type=RZ_ZONE,continent=RZ_EASTERN},
        {name=Z["Tirisfal Glades"],low=1,high=12,faction=RZ_HORDE,type=RZ_ZONE,continent=RZ_EASTERN},
        {name=Z["Loch Modan"],low=10,high=20,faction=RZ_ALLIANCE,type=RZ_ZONE,continent=RZ_EASTERN},
        {name=Z["Westfall"],low=10,high=20,faction=RZ_ALLIANCE,type=RZ_ZONE,continent=RZ_EASTERN},
        {name=Z["Silverpine Forest"],low=10,high=20,faction=RZ_HORDE,type=RZ_ZONE,continent=RZ_EASTERN},
        {name=Z["Redridge Mountains"],low=15,high=25,faction=RZ_CONTESTED,type=RZ_ZONE,continent=RZ_EASTERN},
        {name=Z["Wetlands"],low=20,high=30,faction=RZ_CONTESTED,type=RZ_ZONE,continent=RZ_EASTERN},
        {name=Z["Duskwood"], low=20,high=30,faction=RZ_CONTESTED,type=RZ_ZONE,continent=RZ_EASTERN},
        {name=Z["Hillsbrad Foothills"],low=20,high=30,faction=RZ_CONTESTED,type=RZ_ZONE,continent=RZ_EASTERN},
        {name=Z["Alterac Mountains"],low=30,high=40,faction=RZ_CONTESTED,type=RZ_ZONE,continent=RZ_EASTERN},
        {name=Z["Arathi Highlands"],low=30,high=40,faction=RZ_CONTESTED,type=RZ_ZONE,continent=RZ_EASTERN},
        {name=Z["Stranglethorn Vale"],low=30,high=45,faction=RZ_CONTESTED,type=RZ_ZONE,continent=RZ_EASTERN},
        {name=Z["Badlands"],low=35,high=45,faction=RZ_CONTESTED,type=RZ_ZONE,continent=RZ_EASTERN},
        {name=Z["Swamp of Sorrows"],low=35,high=45,faction=RZ_CONTESTED,type=RZ_ZONE,continent=RZ_EASTERN},
        {name=Z["The Hinterlands"],low=40,high=50,faction=RZ_CONTESTED,type=RZ_ZONE,continent=RZ_EASTERN},
        {name=Z["Searing Gorge"],low=43,high=50,faction=RZ_CONTESTED,type=RZ_ZONE,continent=RZ_EASTERN},
        {name=Z["Blasted Lands"],low=47,high=55,faction=RZ_CONTESTED,type=RZ_ZONE,continent=RZ_EASTERN},
        {name=Z["Burning Steppes"],low=50,high=60,faction=RZ_CONTESTED,type=RZ_ZONE,continent=RZ_EASTERN},
        {name=Z["Western Plaguelands"],low=50,high=60,faction=RZ_CONTESTED,type=RZ_ZONE,continent=RZ_EASTERN},
        {name=Z["Deadwind Pass"],low=55,high=60,faction=RZ_CONTESTED,type=RZ_ZONE,continent=RZ_EASTERN},
        {name=Z["Eastern Plaguelands"],low=55,high=60,faction=RZ_CONTESTED,type=RZ_ZONE,continent=RZ_EASTERN},
        {name=Z["Isle of Quel'Danas"],low=65,high=70,faction=RZ_CONTESTED,type=RZ_ZONE,continent=RZ_EASTERN},

-- Kalimdor
        {name=Z["Darnassus"],low=0,high=0,faction=RZ_ALLIANCE,type=RZ_CITY,continent=RZ_KALIMDOR},
        {name=Z["Orgrimmar"],low=0,high=0,faction=RZ_HORDE,type=RZ_CITY,continent=RZ_KALIMDOR},
        {name=Z["Thunder Bluff"],low=0,high=0,faction=RZ_HORDE,type=RZ_CITY,continent=RZ_KALIMDOR},
        {name=Z["Moonglade"],low=0,high=0,faction=RZ_CONTESTED,type=RZ_CITY,continent=RZ_KALIMDOR},
        {name=Z["Teldrassil"],low=1,high=12,faction=RZ_ALLIANCE,type=RZ_ZONE,continent=RZ_KALIMDOR},
        {name=Z["Durotar"],low=1,high=12,faction=RZ_HORDE,type=RZ_ZONE,continent=RZ_KALIMDOR},
        {name=Z["Mulgore"],low=1,high=12,faction=RZ_HORDE,type=RZ_ZONE,continent=RZ_KALIMDOR},
        {name=Z["Darkshore"],low=10,high=20,faction=RZ_ALLIANCE,type=RZ_ZONE,continent=RZ_KALIMDOR},
        {name=Z["The Barrens"],low=10,high=25,faction=RZ_HORDE,type=RZ_ZONE,continent=RZ_KALIMDOR},
        {name=Z["Stonetalon Mountains"],low=15,high=27,faction=RZ_CONTESTED,type=RZ_ZONE,continent=RZ_KALIMDOR},
        {name=Z["Ashenvale"], low=15,high=30,faction=RZ_CONTESTED,type=RZ_ZONE,continent=RZ_KALIMDOR},
        {name=Z["Thousand Needles"],low=25,high=35,faction=RZ_CONTESTED,type=RZ_ZONE,continent=RZ_KALIMDOR},
        {name=Z["Desolace"],low=30,high=40,faction=RZ_CONTESTED,type=RZ_ZONE,continent=RZ_KALIMDOR},
        {name=Z["Dustwallow Marsh"],low=35,high=45,faction=RZ_CONTESTED,type=RZ_ZONE,continent=RZ_KALIMDOR},
        {name=Z["Feralas"],low=40,high=50,faction=RZ_CONTESTED,type=RZ_ZONE,continent=RZ_KALIMDOR},
        {name=Z["Tanaris"],low=40,high=50,faction=RZ_CONTESTED,type=RZ_ZONE,continent=RZ_KALIMDOR},
        {name=Z["Un'Goro Crater"],low=48,high=55,faction=RZ_CONTESTED,type=RZ_ZONE,continent=RZ_KALIMDOR},
        {name=Z["Azshara"],low=48,high=55,faction=RZ_CONTESTED,type=RZ_ZONE,continent=RZ_KALIMDOR},
        {name=Z["Felwood"],low=48,high=55,faction=RZ_CONTESTED,type=RZ_ZONE,continent=RZ_KALIMDOR},
        {name=Z["Silithus"],low=55,high=60,faction=RZ_CONTESTED,type=RZ_ZONE,continent=RZ_KALIMDOR},
        {name=Z["Winterspring"],low=55,high=60,faction=RZ_CONTESTED,type=RZ_ZONE,continent=RZ_KALIMDOR},

-- Blood Elf
        {name=Z["Silvermoon City"],low=0,high=0,faction=RZ_HORDE,type=RZ_CITY,continent=RZ_EASTERN},
        {name=Z["Eversong Woods"],low=1,high=12,faction=RZ_HORDE,type=RZ_ZONE,continent=RZ_EASTERN},
        {name=Z["Ghostlands"],low=10,high=20,faction=RZ_HORDE,type=RZ_ZONE,continent=RZ_EASTERN},
    
-- Draenei    
        {name=Z["The Exodar"],low=0,high=0,faction=RZ_ALLIANCE,type=RZ_CITY,continent=RZ_KALIMDOR},
        {name=Z["Azuremyst Isle"],low=1,high=12,faction=RZ_ALLIANCE,type=RZ_ZONE,continent=RZ_KALIMDOR},
        {name=Z["Bloodmyst Isle"],low=10,high=20,faction=RZ_ALLIANCE,type=RZ_ZONE,continent=RZ_KALIMDOR},

-- Outland    
        {name=Z["Shattrath City"],low=0,high=0,faction=RZ_SANCTUARY,type=RZ_CITY,continent=RZ_OUTLAND},
        {name=Z["Hellfire Peninsula"],low=58,high=63,faction=RZ_CONTESTED,type=RZ_ZONE,continent=RZ_OUTLAND},
        {name=Z["Zangarmarsh"],low=60,high=64,faction=RZ_CONTESTED,type=RZ_ZONE,continent=RZ_OUTLAND},
        {name=Z["Terokkar Forest"],low=62,high=65,faction=RZ_CONTESTED,type=RZ_ZONE,continent=RZ_OUTLAND},
        {name=Z["Nagrand"],low=64,high=67,faction=RZ_CONTESTED,type=RZ_ZONE,continent=RZ_OUTLAND},
        {name=Z["Blade's Edge Mountains"],low=65,high=68,faction=RZ_CONTESTED,type=RZ_ZONE,continent=RZ_OUTLAND},
        {name=Z["Netherstorm"],low=67,high=70,faction=RZ_CONTESTED,type=RZ_ZONE,continent=RZ_OUTLAND},
        {name=Z["Shadowmoon Valley"],low=67,high=70,faction=RZ_CONTESTED,type=RZ_ZONE,continent=RZ_OUTLAND},

-- Northrend
        {name=Z["Borean Tundra"],low=68,high=72,faction=RZ_CONTESTED,type=RZ_ZONE,continent=RZ_NORTHREND},
        {name=Z["Howling Fjord"],low=68,high=72,faction=RZ_CONTESTED,type=RZ_ZONE,continent=RZ_NORTHREND},
        {name=Z["Dragonblight"],low=71,high=74,faction=RZ_CONTESTED,type=RZ_ZONE,continent=RZ_NORTHREND},
        {name=Z["Grizzly Hills"],low=73,high=75,faction=RZ_CONTESTED,type=RZ_ZONE,continent=RZ_NORTHREND},
        {name=Z["Zul'Drak"],low=74,high=77,faction=RZ_CONTESTED,type=RZ_ZONE,continent=RZ_NORTHREND},
        {name=Z["Sholazar Basin"],low=75,high=78,faction=RZ_CONTESTED,type=RZ_ZONE,continent=RZ_NORTHREND},
        {name=Z["Crystalsong Forest"],low=77,high=80,faction=RZ_CONTESTED,type=RZ_ZONE,continent=RZ_NORTHREND},
        {name=Z["The Storm Peaks"],low=76,high=80,faction=RZ_CONTESTED,type=RZ_ZONE,continent=RZ_NORTHREND},
        {name=Z["Icecrown"],low=77,high=80,faction=RZ_CONTESTED,type=RZ_ZONE,continent=RZ_NORTHREND},
        {name=Z["Wintergrasp"],low=77,high=80,faction=RZ_CONTESTED,type=RZ_PVP,continent=RZ_NORTHREND},
        {name=Z["Dalaran"],low=0,high=0,faction=RZ_SANCTUARY,type=RZ_CITY,continent=RZ_NORTHREND},

-- Death Knights
--        {name=Z["Acherus: The Ebon Hold"],low=0,high=0,faction=RZ_SANCTUARY,type=RZ_CITY,continent=RZ_EASTERN},

-- Instances
        {name=Z["Ragefire Chasm"],low=13,high=20,type=RZ_INSTANCE,faction=RZ_HORDE,continent=RZ_KALIMDOR},
        {name=Z["Wailing Caverns"],low=17,high=24,type=RZ_INSTANCE,faction=RZ_CONTESTED,continent=RZ_KALIMDOR},
        {name=Z["The Deadmines"],low=17,high=26,type=RZ_INSTANCE,faction=RZ_CONTESTED,continent=RZ_EASTERN},
        {name=Z["The Stockade"],low=24,high=32,type=RZ_INSTANCE,faction=RZ_ALLIANCE,continent=RZ_EASTERN},
        {name=Z["Shadowfang Keep"],low=20,high=28,type=RZ_INSTANCE,faction=RZ_CONTESTED,continent=RZ_EASTERN},
        {name=Z["Blackfathom Deeps"],low=23,high=30,type=RZ_INSTANCE,faction=RZ_CONTESTED,continent=RZ_KALIMDOR},
        {name=Z["Gnomeregan"],low=29,high=38,type=RZ_INSTANCE,faction=RZ_CONTESTED,continent=RZ_EASTERN},
        {name=Z["Razorfen Kraul"],low=29,high=38,type=RZ_INSTANCE,faction=RZ_CONTESTED,continent=RZ_KALIMDOR},
        {name=Z["Scarlet Monastery"],low=33,high=44,type=RZ_INSTANCE,faction=RZ_CONTESTED,continent=RZ_EASTERN},
        {name=Z["Razorfen Downs"],low=37,high=46,type=RZ_INSTANCE,faction=RZ_CONTESTED,continent=RZ_KALIMDOR},
        {name=Z["Uldaman"],low=38,high=46,type=RZ_INSTANCE,faction=RZ_CONTESTED,continent=RZ_EASTERN},
        {name=Z["Maraudon"],low=45,high=55,type=RZ_INSTANCE,faction=RZ_CONTESTED,continent=RZ_KALIMDOR},
        {name=Z["Zul'Farrak"],low=43,high=50,type=RZ_INSTANCE,faction=RZ_CONTESTED,continent=RZ_KALIMDOR},
        {name=Z["The Temple of Atal'Hakkar"],low=44,high=52,type=RZ_INSTANCE,faction=RZ_CONTESTED,continent=RZ_EASTERN},
        {name=Z["Blackrock Depths"],low=50,high=56,type=RZ_INSTANCE,faction=RZ_CONTESTED,continent=RZ_EASTERN},
        {name=Z["Blackrock Spire"],low=56,high=60,type=RZ_INSTANCE,faction=RZ_CONTESTED,continent=RZ_EASTERN},
        {name=Z["Stratholme"],low=56,high=60,type=RZ_INSTANCE,faction=RZ_CONTESTED,continent=RZ_EASTERN},
        {name=Z["Dire Maul"],low=56,high=60,type=RZ_INSTANCE,faction=RZ_CONTESTED,continent=RZ_KALIMDOR},
        {name=Z["Scholomance"],low=58,high=60,type=RZ_INSTANCE,faction=RZ_CONTESTED,continent=RZ_EASTERN},
        {name=Z["Warsong Gulch"],x=1,low=10,high=-1,type=RZ_BATTLEGROUND,faction=RZ_HORDE,continent=RZ_KALIMDOR},
        {name=Z["Warsong Gulch"],x=2,low=10,high=-1,type=RZ_BATTLEGROUND,faction=RZ_ALLIANCE,continent=RZ_KALIMDOR},
        {name=Z["Arathi Basin"],low=20,high=-1,type=RZ_BATTLEGROUND,faction=RZ_CONTESTED,continent=RZ_EASTERN},
        {name=Z["Alterac Valley"],low=51,high=-1,type=RZ_BATTLEGROUND,faction=RZ_CONTESTED,continent=RZ_EASTERN},
        {name=Z["Blackwing Lair"],low=60,high=60,type=RZ_RAID40,faction=RZ_CONTESTED,continent=RZ_EASTERN},
        {name=Z["Molten Core"],low=60,high=60,type=RZ_RAID40,faction=RZ_CONTESTED,continent=RZ_EASTERN},
        {name=Z["Onyxia's Lair"],low=60,high=60,type=RZ_RAID40,faction=RZ_CONTESTED,continent=RZ_KALIMDOR},
        {name=Z["Zul'Gurub"],low=60,high=60,type=RZ_RAID20,faction=RZ_CONTESTED,continent=RZ_EASTERN},
        {name=Z["Ahn'Qiraj"],low=60,high=60,type=RZ_RAID20,faction=RZ_CONTESTED,continent=RZ_KALIMDOR},
        {name=Z["Temple of Ahn'Qiraj"],low=60,high=60,type=RZ_RAID40,faction=RZ_CONTESTED,continent=RZ_EASTERN},

-- BC Instances
        {name=Z["Hellfire Ramparts"],low=60,high=62,faction=RZ_CONTESTED,type=RZ_INSTANCE,continent=RZ_OUTLAND},
        {name=Z["The Blood Furnace"],low=61,high=63,faction=RZ_CONTESTED,type=RZ_INSTANCE,continent=RZ_OUTLAND},
        {name=Z["The Slave Pens"],low=62,high=64,faction=RZ_CONTESTED,type=RZ_INSTANCE,continent=RZ_OUTLAND},
        {name=Z["The Underbog"],low=63,high=65,faction=RZ_CONTESTED,type=RZ_INSTANCE,continent=RZ_OUTLAND},
        {name=Z["Auchenai Crypts"],low=64,high=66,faction=RZ_CONTESTED,type=RZ_INSTANCE,continent=RZ_OUTLAND},
        {name=Z["Shadow Labyrinth"],low=65,high=67,faction=RZ_CONTESTED,type=RZ_INSTANCE,continent=RZ_OUTLAND},
        {name=Z["Sethekk Halls"],low=67,high=69,faction=RZ_CONTESTED,type=RZ_INSTANCE,continent=RZ_OUTLAND},
        {name=Z["The Mechanar"],low=69,high=72,faction=RZ_CONTESTED,type=RZ_INSTANCE,continent=RZ_OUTLAND},
        {name=Z["The Shattered Halls"],low=70,high=72,faction=RZ_CONTESTED,type=RZ_INSTANCE,continent=RZ_OUTLAND},
        {name=Z["The Botanica"],low=70,high=72,faction=RZ_CONTESTED,type=RZ_INSTANCE,continent=RZ_OUTLAND},
        {name=Z["The Arcatraz"],low=70,high=72,faction=RZ_CONTESTED,type=RZ_INSTANCE,continent=RZ_OUTLAND},
        {name=Z["The Eye"],low=70,high=72,faction=RZ_CONTESTED,type=RZ_RAID25,continent=RZ_OUTLAND},
        {name=Z["Mana-Tombs"],low=70,high=72,faction=RZ_CONTESTED,type=RZ_INSTANCE,continent=RZ_OUTLAND},
        {name=Z["The Steamvault"],low=70,high=72,faction=RZ_CONTESTED,type=RZ_INSTANCE,continent=RZ_OUTLAND},
        {name=Z["Karazhan"],low=70,high=72,faction=RZ_CONTESTED,type=RZ_RAID10,continent=RZ_EASTERN},
        {name=Z["Gruul's Lair"],low=70,high=72,faction=RZ_CONTESTED,type=RZ_RAID25,continent=RZ_OUTLAND},
        {name=Z["Magtheridon's Lair"],low=70,high=72,faction=RZ_CONTESTED,type=RZ_RAID25,continent=RZ_OUTLAND},
        {name=Z["Eye of the Storm"],low=61,high=-1,faction=RZ_CONTESTED,type=RZ_BATTLEGROUND,continent=RZ_OUTLAND},
        {name=Z["Black Temple"],low=70,high=72,faction=RZ_CONTESTED,type=RZ_RAID25,continent=RZ_OUTLAND},
        {name=Z["Serpentshrine Cavern"],low=70,high=72,faction=RZ_CONTESTED,type=RZ_RAID25,continent=RZ_OUTLAND},
        {name=Z["Magisters' Terrace"],low=70,high=72,faction=RZ_CONTESTED,type=RZ_INSTANCE,continent=RZ_EASTERN},
        {name=Z["Sunwell Plateau"],low=70,high=72,faction=RZ_CONTESTED,type=RZ_RAID25,continent=RZ_EASTERN},
        {name=Z["Zul'Aman"],low=70,high=72,faction=RZ_CONTESTED,type=RZ_RAID10,continent=RZ_EASTERN},
        {name=Z["Old Hillsbrad Foothills"],low=66,high=68,faction=RZ_CONTESTED,type=RZ_INSTANCE,continent=RZ_KALIMDOR},
        {name=Z["The Black Morass"],low=68,high=70,faction=RZ_CONTESTED,type=RZ_INSTANCE,continent=RZ_KALIMDOR},
        {name=Z["Hyjal Summit"],low=70,high=72,faction=RZ_CONTESTED,type=RZ_RAID25,continent=RZ_KALIMDOR},

-- WotLK Instances
        {name=Z["Utgarde Keep"],low=70,high=72,type=RZ_INSTANCE,faction=RZ_CONTESTED,continent=RZ_NORTHREND},
        {name=Z["Utgarde Pinnacle"],low=80,high=82,type=RZ_INSTANCE,faction=RZ_CONTESTED,continent=RZ_NORTHREND},
        {name=Z["The Nexus"],low=71,high=73,type=RZ_INSTANCE,faction=RZ_CONTESTED,continent=RZ_NORTHREND},
        {name=Z["The Oculus"],low=80,high=82,type=RZ_INSTANCE,faction=RZ_CONTESTED,continent=RZ_NORTHREND},
        {name=Z["The Nexus"],low=80,high=82,type=RZ_RAID10_20,faction=RZ_CONTESTED,continent=RZ_NORTHREND},
        {name=Z["Azjol-Nerub"],low=72,high=74,type=RZ_INSTANCE,faction=RZ_CONTESTED,continent=RZ_NORTHREND},
        {name=Z["Ahn'kahet: The Old Kingdom"],low=73,high=75,type=RZ_INSTANCE,faction=RZ_CONTESTED,continent=RZ_NORTHREND},
        {name=Z["Drak'Tharon Keep"],low=74,high=76,type=RZ_INSTANCE,faction=RZ_CONTESTED,continent=RZ_NORTHREND},
        {name=Z["The Violet Hold"],low=75,high=77,type=RZ_INSTANCE,faction=RZ_CONTESTED,continent=RZ_NORTHREND},
        {name=Z["Gundrak"],low=76,high=78,type=RZ_INSTANCE,faction=RZ_CONTESTED,continent=RZ_NORTHREND},
        {name=Z["Halls of Stone"],low=77,high=79,type=RZ_INSTANCE,faction=RZ_CONTESTED,continent=RZ_NORTHREND},
        {name=Z["Halls of Lightning"],low=80,high=82,type=RZ_INSTANCE,faction=RZ_CONTESTED,continent=RZ_NORTHREND},
        {name=Z["The Eye of Eternity"],low=80,high=82,type=RZ_RAID10_20,faction=RZ_CONTESTED,continent=RZ_NORTHREND},
        {name=Z["The Obsidian Sanctum"],low=80,high=82,type=RZ_RAID10_20,faction=RZ_CONTESTED,continent=RZ_NORTHREND},
        {name=Z["Naxxramas"],low=80,high=82,type=RZ_RAID10_20,faction=RZ_CONTESTED,continent=RZ_NORTHREND},
        {name=Z["Strand of the Ancients"],low=71,high=-1,type=RZ_BATTLEGROUND,faction=RZ_CONTESTED,continent=RZ_NORTHREND},
        {name=Z["The Culling of Stratholme"],low=80,high=82,type=RZ_INSTANCE,faction=RZ_CONTESTED,continent=RZ_KALIMDOR},

-- Towns and transport zones
        {name=Z["Booty Bay"],low=30,high=45,faction=RZ_CONTESTED,type=RZ_CITY,continent=RZ_EASTERN},
        {name=Z["Ratchet"],low=10,high=25,faction=RZ_CONTESTED,type=RZ_CITY,continent=RZ_KALIMDOR},
        {name=Z["Gadgetzan"],low=40,high=50,faction=RZ_CONTESTED,type=RZ_CITY,continent=RZ_KALIMDOR},
        {name=Z["Deeprun Tram"],low=0,high=0,faction=RZ_ALLIANCE,type=RZ_CITY,continent=RZ_EASTERN},
        {name=Z["Theramore Isle"],low=35,high=45,faction=RZ_ALLIANCE,type=RZ_CITY,continent=RZ_KALIMDOR},
        {name=Z["Menethil Harbor"],low=20,high=30,faction=RZ_ALLIANCE,type=RZ_CITY,continent=RZ_EASTERN},
        {name=Z["Auberdine"],low=10,high=20,faction=RZ_ALLIANCE,type=RZ_CITY,continent=RZ_KALIMDOR},
        {name=Z["Grom'gol Base Camp"],low=30,high=45,faction=RZ_HORDE,type=RZ_CITY,continent=RZ_EASTERN},
        {name=Z["Blackrock Mountain"],low=60,high=60,type=RZ_TRANSPORT,faction=RZ_CONTESTED,continent=RZ_EASTERN},

-- Transports
        {name=L["Boat"],x=1,low=0,high=0,faction=RZ_ALLIANCE,type=RZ_TRANSPORT,continent=RZ_OTHER},
        {name=L["Boat"],x=2,low=0,high=0,faction=RZ_ALLIANCE,type=RZ_TRANSPORT,continent=RZ_OTHER},
        {name=L["Boat"],x=3,low=0,high=0,faction=RZ_ALLIANCE,type=RZ_TRANSPORT,continent=RZ_OTHER},
        {name=L["Boat"],x=4,low=0,high=0,faction=RZ_CONTESTED,type=RZ_TRANSPORT,continent=RZ_OTHER},
        {name=L["Boat"],x=5,low=0,high=0,faction=RZ_ALLIANCE,type=RZ_TRANSPORT,continent=RZ_OTHER},
        {name=L["Boat"],x=6,low=0,high=0,faction=RZ_ALLIANCE,type=RZ_TRANSPORT,continent=RZ_OTHER},
        {name=L["Boat"],x=7,low=0,high=0,faction=RZ_ALLIANCE,type=RZ_TRANSPORT,continent=RZ_OTHER},
        {name=L["Boat"],x=8,low=0,high=0,faction=RZ_CONTESTED,type=RZ_TRANSPORT,continent=RZ_OTHER},
        {name=L["Boat"],x=9,low=0,high=0,faction=RZ_CONTESTED,type=RZ_TRANSPORT,continent=RZ_OTHER},
        {name=L["Portal"],x=1,low=70,high=-1,faction=RZ_CONTESTED,type=RZ_TRANSPORT,continent=RZ_OTHER},
        {name=L["Portal"],x=2,low=0,high=0,faction=RZ_ALLIANCE,type=RZ_TRANSPORT,continent=RZ_OTHER},
        {name=L["Portal"],x=3,low=0,high=0,faction=RZ_ALLIANCE,type=RZ_TRANSPORT,continent=RZ_OTHER},
        {name=L["Portal"],x=4,low=0,high=0,faction=RZ_ALLIANCE,type=RZ_TRANSPORT,continent=RZ_OTHER},
        {name=L["Portal"],x=5,low=0,high=0,faction=RZ_HORDE,type=RZ_TRANSPORT,continent=RZ_OTHER},
        {name=L["Portal"],x=6,low=0,high=0,faction=RZ_HORDE,type=RZ_TRANSPORT,continent=RZ_OTHER},
        {name=L["Portal"],x=7,low=0,high=0,faction=RZ_HORDE,type=RZ_TRANSPORT,continent=RZ_OTHER},
        {name=L["Portal"],x=8,low=0,high=0,faction=RZ_ALLIANCE,type=RZ_TRANSPORT,continent=RZ_OTHER},
        {name=L["Portal"],x=9,low=0,high=0,faction=RZ_HORDE,type=RZ_TRANSPORT,continent=RZ_OTHER},
        {name=L["Portal"],x=10,low=0,high=0,faction=RZ_ALLIANCE,type=RZ_TRANSPORT,continent=RZ_OTHER},
        {name=L["Portal"],x=11,low=0,high=0,faction=RZ_ALLIANCE,type=RZ_TRANSPORT,continent=RZ_OTHER},
        {name=L["Portal"],x=12,low=0,high=0,faction=RZ_ALLIANCE,type=RZ_TRANSPORT,continent=RZ_OTHER},
        {name=L["Portal"],x=13,low=0,high=0,faction=RZ_ALLIANCE,type=RZ_TRANSPORT,continent=RZ_OTHER},
        {name=L["Portal"],x=14,low=0,high=0,faction=RZ_ALLIANCE,type=RZ_TRANSPORT,continent=RZ_OTHER},
        {name=L["Portal"],x=15,low=0,high=0,faction=RZ_HORDE,type=RZ_TRANSPORT,continent=RZ_OTHER},
        {name=L["Portal"],x=16,low=0,high=0,faction=RZ_HORDE,type=RZ_TRANSPORT,continent=RZ_OTHER},
        {name=L["Portal"],x=17,low=0,high=0,faction=RZ_HORDE,type=RZ_TRANSPORT,continent=RZ_OTHER},
        {name=L["Portal"],x=18,low=0,high=0,faction=RZ_HORDE,type=RZ_TRANSPORT,continent=RZ_OTHER},
        {name=L["Portal"],x=19,low=0,high=0,faction=RZ_HORDE,type=RZ_TRANSPORT,continent=RZ_OTHER},
        {name=L["Zeppelin"],x=1,low=0,high=0,faction=RZ_HORDE,type=RZ_TRANSPORT,continent=RZ_OTHER},
        {name=L["Zeppelin"],x=2,low=0,high=0,faction=RZ_HORDE,type=RZ_TRANSPORT,continent=RZ_OTHER},
        {name=L["Zeppelin"],x=3,low=0,high=0,faction=RZ_HORDE,type=RZ_TRANSPORT,continent=RZ_OTHER},
        {name=L["Zeppelin"],x=4,low=0,high=0,faction=RZ_HORDE,type=RZ_TRANSPORT,continent=RZ_OTHER},
        {name=L["Zeppelin"],x=5,low=0,high=0,faction=RZ_HORDE,type=RZ_TRANSPORT,continent=RZ_OTHER},

-- Transport between old and new world    
        {name=Z["The Dark Portal"],low=58,high=-1,faction=RZ_CONTESTED,type=RZ_TRANSPORT,continent=RZ_OTHER},
        {name=L["Orb of Translocation"],low=0,high=0,faction=RZ_HORDE,type=RZ_TRANSPORT,continent=RZ_OTHER},
    
-- Not real zones
        {name=Z["The Forbidding Sea"],low=0,high=0,faction=RZ_CONTESTED,type=RZ_NONE,continent=RZ_OTHER},
        {name=Z["Twisting Nether"],low=0,high=0,faction=RZ_CONTESTED,type=RZ_NONE,continent=RZ_OTHER},
        {name=Z["The Great Sea"],low=0,high=0,faction=RZ_CONTESTED,type=RZ_NONE,continent=RZ_OTHER},
        {name=Z["The North Sea"],low=0,high=0,faction=RZ_CONTESTED,type=RZ_NONE,continent=RZ_OTHER},
        {name=Z["The Veiled Sea"],low=0,high=0,faction=RZ_CONTESTED,type=RZ_NONE,continent=RZ_OTHER},
        {name=Z["Hall of Legends"],low=0,high=0,faction=RZ_CONTESTED,type=RZ_NONE,continent=RZ_OTHER},
        {name=Z["Hyjal"],low=0,high=0,faction=RZ_CONTESTED,type=RZ_NONE,continent=RZ_KALIMDOR},
        {name=Z["The Frozen Sea"],low=0,high=0,faction=RZ_CONTESTED,type=RZ_NONE,continent=RZ_NORTHREND},
        
-- Arenas
        {name=Z["Blade's Edge Arena"],low=0,high=0,faction=RZ_CONTESTED,type=RZ_ARENA,continent=RZ_OUTLAND},
        {name=Z["Nagrand Arena"],low=0,high=0,faction=RZ_CONTESTED,type=RZ_ARENA,continent=RZ_OUTLAND},
        {name=Z["Ruins of Lordaeron"],low=0,high=0,faction=RZ_CONTESTED,type=RZ_ARENA,continent=RZ_EASTERN},
        {name=Z["Dalaran Sewers"],low=0,high=0,faction=RZ_CONTESTED,type=RZ_ARENA,continent=RZ_NORTHREND},
        {name=Z["The Ring of Valor"],low=0,high=0,faction=RZ_CONTESTED,type=RZ_ARENA,continent=RZ_KALIMDOR},
    };

    table.sort(Broker_RecZone.vars.zones,
        function(a,b)
            if a.high == -1 then a.high = RZ_MAXLEVEL end
            if b.high == -1 then b.high = RZ_MAXLEVEL end
            if a.low<b.low then return true end
            if a.low==b.low and a.high<b.high then return true end
            if a.low==b.low and a.high==b.high and a.name<b.name then return true end
        end);

    Broker_RecZone.vars.zones[zn(Z["Stormwind City"])].zones={zn(Z["Deeprun Tram"]),zn(Z["Elwynn Forest"]),zn(Z["The Stockade"]),znx(L["Boat"],2),znx(L["Boat"],6)}
    Broker_RecZone.vars.zones[zn(Z["Ironforge"])].zones={zn(Z["Deeprun Tram"]),zn(Z["Dun Morogh"]),zn(Z["Isle of Quel'Danas"]),znx(L["Portal"],2)}
    Broker_RecZone.vars.zones[zn(Z["Darnassus"])].zones={zn(Z["Teldrassil"]),znx(L["Boat"],3)}
    Broker_RecZone.vars.zones[zn(Z["Undercity"])].zones={zn(L["Orb of Translocation"]),zn(Z["Tirisfal Glades"])}
    Broker_RecZone.vars.zones[zn(Z["Orgrimmar"])].zones={zn(Z["Durotar"]),zn(Z["Ragefire Chasm"])}
    Broker_RecZone.vars.zones[zn(Z["Thunder Bluff"])].zones={zn(Z["Mulgore"])}
    Broker_RecZone.vars.zones[zn(Z["Moonglade"])].zones={zn(Z["Winterspring"]),zn(Z["Felwood"])}
    Broker_RecZone.vars.zones[zn(Z["Dun Morogh"])].zones={zn(Z["Ironforge"]),zn(Z["Loch Modan"]),zn(Z["Gnomeregan"])}
    Broker_RecZone.vars.zones[zn(Z["Elwynn Forest"])].zones={zn(Z["Stormwind City"]),zn(Z["Westfall"]),zn(Z["Redridge Mountains"]),zn(Z["Duskwood"])}
    Broker_RecZone.vars.zones[zn(Z["Teldrassil"])].zones={zn(Z["Darnassus"])}
    Broker_RecZone.vars.zones[zn(Z["Tirisfal Glades"])].zones={zn(Z["Undercity"]),zn(Z["Silverpine Forest"]),zn(Z["Western Plaguelands"]),znx(L["Zeppelin"],1),znx(L["Zeppelin"],2),zn(Z["Scarlet Monastery"]),znx(L["Zeppelin"],5)}
    Broker_RecZone.vars.zones[zn(Z["Durotar"])].zones={zn(Z["Orgrimmar"]),zn(Z["The Barrens"]),zn(Z["Azshara"]),znx(L["Zeppelin"],1),znx(L["Zeppelin"],3),znx(L["Zeppelin"],4)}
    Broker_RecZone.vars.zones[zn(Z["Mulgore"])].zones={zn(Z["Thunder Bluff"]),zn(Z["The Barrens"])}
    Broker_RecZone.vars.zones[zn(Z["Loch Modan"])].zones={zn(Z["Dun Morogh"]),zn(Z["Wetlands"]),zn(Z["Badlands"])}
    Broker_RecZone.vars.zones[zn(Z["Westfall"])].zones={zn(Z["Elwynn Forest"]),zn(Z["Duskwood"]),zn(Z["The Deadmines"])}
    Broker_RecZone.vars.zones[zn(Z["Silverpine Forest"])].zones={zn(Z["Tirisfal Glades"]),zn(Z["Hillsbrad Foothills"]),zn(Z["Alterac Mountains"]),zn(Z["Shadowfang Keep"])}
    Broker_RecZone.vars.zones[zn(Z["Darkshore"])].zones={zn(Z["Ashenvale"]),zn(Z["Auberdine"])}
    Broker_RecZone.vars.zones[zn(Z["The Barrens"])].zones={zn(Z["Ratchet"]),zn(Z["Durotar"]),zn(Z["Mulgore"]),zn(Z["Stonetalon Mountains"]),zn(Z["Ashenvale"]),zn(Z["Thousand Needles"]),zn(Z["Dustwallow Marsh"]),zn(Z["Wailing Caverns"]),zn(Z["Razorfen Kraul"]),zn(Z["Razorfen Downs"]),znx(Z["Warsong Gulch"],1)}
    Broker_RecZone.vars.zones[zn(Z["Redridge Mountains"])].zones={zn(Z["Elwynn Forest"]),zn(Z["Burning Steppes"])}
    Broker_RecZone.vars.zones[zn(Z["Stonetalon Mountains"])].zones={zn(Z["The Barrens"]),zn(Z["Ashenvale"]),zn(Z["Desolace"])}
    Broker_RecZone.vars.zones[zn(Z["Ashenvale"])].zones={zn(Z["Darkshore"]),zn(Z["The Barrens"]),zn(Z["Stonetalon Mountains"]),zn(Z["Azshara"]),zn(Z["Felwood"]),zn(Z["Blackfathom Deeps"]),znx(Z["Warsong Gulch"],2)}
    Broker_RecZone.vars.zones[zn(Z["Wetlands"])].zones={zn(Z["Loch Modan"]),zn(Z["Arathi Highlands"]),zn(Z["Menethil Harbor"])}
    Broker_RecZone.vars.zones[zn(Z["Duskwood"])].zones={zn(Z["Elwynn Forest"]),zn(Z["Westfall"]),zn(Z["Stranglethorn Vale"]),zn(Z["Deadwind Pass"])}
    Broker_RecZone.vars.zones[zn(Z["Hillsbrad Foothills"])].zones={zn(Z["Silverpine Forest"]),zn(Z["Alterac Mountains"]),zn(Z["Arathi Highlands"])}
    Broker_RecZone.vars.zones[zn(Z["Thousand Needles"])].zones={zn(Z["The Barrens"]),zn(Z["Feralas"]),zn(Z["Tanaris"])}
    Broker_RecZone.vars.zones[zn(Z["Alterac Mountains"])].zones={zn(Z["Silverpine Forest"]),zn(Z["Hillsbrad Foothills"]),zn(Z["The Hinterlands"]),zn(Z["Western Plaguelands"]),zn(Z["Alterac Valley"])}
    Broker_RecZone.vars.zones[zn(Z["Arathi Highlands"])].zones={zn(Z["Wetlands"]),zn(Z["Hillsbrad Foothills"]),zn(Z["Arathi Basin"])}
    Broker_RecZone.vars.zones[zn(Z["Desolace"])].zones={zn(Z["Stonetalon Mountains"]),zn(Z["Feralas"]),zn(Z["Maraudon"])}
    Broker_RecZone.vars.zones[zn(Z["Stranglethorn Vale"])].zones={zn(Z["Booty Bay"]),zn(Z["Duskwood"]),zn(Z["Grom'gol Base Camp"]),zn(Z["Zul'Gurub"])}
    Broker_RecZone.vars.zones[zn(Z["Badlands"])].zones={zn(Z["Loch Modan"]),zn(Z["Searing Gorge"]),zn(Z["Uldaman"])}
    Broker_RecZone.vars.zones[zn(Z["Swamp of Sorrows"])].zones={zn(Z["Blasted Lands"]),zn(Z["Deadwind Pass"]),zn(Z["The Temple of Atal'Hakkar"])}
    Broker_RecZone.vars.zones[zn(Z["Dustwallow Marsh"])].zones={zn(Z["The Barrens"]),zn(Z["Theramore Isle"]),zn(Z["Onyxia's Lair"])}
    Broker_RecZone.vars.zones[zn(Z["The Hinterlands"])].zones={zn(Z["Alterac Mountains"]),zn(Z["Western Plaguelands"])}
    Broker_RecZone.vars.zones[zn(Z["Feralas"])].zones={zn(Z["Thousand Needles"]),zn(Z["Desolace"]),zn(Z["Dire Maul"])}
    Broker_RecZone.vars.zones[zn(Z["Tanaris"])].zones={zn(Z["Thousand Needles"]),zn(Z["Un'Goro Crater"]),zn(Z["Gadgetzan"]),zn(Z["Zul'Farrak"]),zn(Z["Old Hillsbrad Foothills"]),zn(Z["The Black Morass"]),zn(Z["Hyjal Summit"]),zn(Z["The Culling of Stratholme"])}
    Broker_RecZone.vars.zones[zn(Z["Searing Gorge"])].zones={zn(Z["Badlands"]),zn(Z["Blackrock Mountain"])}
    Broker_RecZone.vars.zones[zn(Z["Blasted Lands"])].zones={zn(Z["Swamp of Sorrows"]),zn(Z["The Dark Portal"])}
    Broker_RecZone.vars.zones[zn(Z["Un'Goro Crater"])].zones={zn(Z["Tanaris"]),zn(Z["Silithus"])}
    Broker_RecZone.vars.zones[zn(Z["Azshara"])].zones={zn(Z["Durotar"]),zn(Z["Ashenvale"])}
    Broker_RecZone.vars.zones[zn(Z["Felwood"])].zones={zn(Z["Darkshore"]),zn(Z["Ashenvale"]),zn(Z["Winterspring"]),zn(Z["Moonglade"])}
    Broker_RecZone.vars.zones[zn(Z["Burning Steppes"])].zones={zn(Z["Redridge Mountains"]),zn(Z["Blackrock Mountain"])}
    Broker_RecZone.vars.zones[zn(Z["Western Plaguelands"])].zones={zn(Z["Tirisfal Glades"]),zn(Z["Alterac Mountains"]),zn(Z["The Hinterlands"]),zn(Z["Eastern Plaguelands"]),zn(Z["Scholomance"])}
    Broker_RecZone.vars.zones[zn(Z["Deadwind Pass"])].zones={zn(Z["Duskwood"]),zn(Z["Swamp of Sorrows"]),zn(Z["Karazhan"])}
    Broker_RecZone.vars.zones[zn(Z["Karazhan"])].zones={zn(Z["Deadwind Pass"])}
    Broker_RecZone.vars.zones[zn(Z["Eastern Plaguelands"])].zones={zn(Z["Western Plaguelands"]),zn(Z["Stratholme"]),zn(Z["Ghostlands"])}
    Broker_RecZone.vars.zones[zn(Z["Silithus"])].zones={zn(Z["Un'Goro Crater"]),zn(Z["Ahn'Qiraj"]),zn(Z["Temple of Ahn'Qiraj"])}
    Broker_RecZone.vars.zones[zn(Z["Winterspring"])].zones={zn(Z["Moonglade"]),zn(Z["Felwood"])}

    Broker_RecZone.vars.zones[zn(Z["Isle of Quel'Danas"])].zones={zn(Z["Magisters' Terrace"]),zn(Z["Sunwell Plateau"]),zn(Z["Ironforge"]),zn(Z["Silvermoon City"])}

    Broker_RecZone.vars.zones[zn(Z["Silvermoon City"])].zones={zn(Z["Eversong Woods"]),zn(L["Orb of Translocation"]),zn(Z["Isle of Quel'Danas"])}
    Broker_RecZone.vars.zones[zn(Z["Eversong Woods"])].zones={zn(Z["Ghostlands"]),zn(Z["Silvermoon City"])}
    Broker_RecZone.vars.zones[zn(Z["Ghostlands"])].zones={zn(Z["Eversong Woods"]),zn(Z["Eastern Plaguelands"]),zn(Z["Zul'Aman"])}

    Broker_RecZone.vars.zones[zn(Z["Auberdine"])].zones={zn(Z["Darkshore"]),znx(L["Boat"],2),znx(L["Boat"],3),znx(L["Boat"],5)}
    Broker_RecZone.vars.zones[zn(Z["Azuremyst Isle"])].zones={zn(Z["Bloodmyst Isle"]),zn(Z["The Exodar"]),znx(L["Boat"],5)}
    Broker_RecZone.vars.zones[zn(Z["Bloodmyst Isle"])].zones={zn(Z["Azuremyst Isle"])}
    Broker_RecZone.vars.zones[zn(Z["The Exodar"])].zones={zn(Z["Azuremyst Isle"])}

    Broker_RecZone.vars.zones[zn(Z["Shattrath City"])].zones={zn(Z["Terokkar Forest"]),znx(L["Portal"],1),znx(L["Portal"],2),znx(L["Portal"],3),znx(L["Portal"],4),znx(L["Portal"],5),znx(L["Portal"],6),znx(L["Portal"],7),znx(L["Portal"],8),znx(L["Portal"],9)}
    Broker_RecZone.vars.zones[zn(Z["Hellfire Peninsula"])].zones={zn(Z["Zangarmarsh"]),zn(Z["Terokkar Forest"]),zn(Z["Hellfire Ramparts"]),zn(Z["The Blood Furnace"]),zn(Z["The Shattered Halls"]),zn(Z["Magtheridon's Lair"]),zn(Z["The Dark Portal"])}
    Broker_RecZone.vars.zones[zn(Z["Zangarmarsh"])].zones={zn(Z["Hellfire Peninsula"]),zn(Z["Terokkar Forest"]),zn(Z["Nagrand"]),zn(Z["Blade's Edge Mountains"]),zn(Z["The Slave Pens"]),zn(Z["The Underbog"]),zn(Z["The Steamvault"]),zn(Z["Serpentshrine Cavern"])}
    Broker_RecZone.vars.zones[zn(Z["Terokkar Forest"])].zones={zn(Z["Hellfire Peninsula"]),zn(Z["Zangarmarsh"]),zn(Z["Nagrand"]),zn(Z["Shadowmoon Valley"]),zn(Z["Auchenai Crypts"]),zn(Z["Shadow Labyrinth"]),zn(Z["Sethekk Halls"]),zn(Z["Mana-Tombs"]),zn(Z["Shattrath City"])}
    Broker_RecZone.vars.zones[zn(Z["Nagrand"])].zones={zn(Z["Zangarmarsh"]),zn(Z["Terokkar Forest"])}
    Broker_RecZone.vars.zones[zn(Z["Blade's Edge Mountains"])].zones={zn(Z["Zangarmarsh"]),zn(Z["Netherstorm"]),zn(Z["Gruul's Lair"])}
    Broker_RecZone.vars.zones[zn(Z["Netherstorm"])].zones={zn(Z["Blade's Edge Mountains"]),zn(Z["The Mechanar"]),zn(Z["The Botanica"]),zn(Z["The Arcatraz"]),zn(Z["The Eye"])}
    Broker_RecZone.vars.zones[zn(Z["Shadowmoon Valley"])].zones={zn(Z["Terokkar Forest"]),zn(Z["Black Temple"])}
    
    Broker_RecZone.vars.zones[zn(Z["Ragefire Chasm"])].zones={zn(Z["Orgrimmar"])}
    Broker_RecZone.vars.zones[zn(Z["Wailing Caverns"])].zones={zn(Z["The Barrens"])}
    Broker_RecZone.vars.zones[zn(Z["The Deadmines"])].zones={zn(Z["Westfall"])}
    Broker_RecZone.vars.zones[zn(Z["The Stockade"])].zones={zn(Z["Stormwind City"])}
    Broker_RecZone.vars.zones[zn(Z["Shadowfang Keep"])].zones={zn(Z["Silverpine Forest"])}
    Broker_RecZone.vars.zones[zn(Z["Blackfathom Deeps"])].zones={zn(Z["Ashenvale"])}
    Broker_RecZone.vars.zones[zn(Z["Gnomeregan"])].zones={zn(Z["Dun Morogh"])}
    Broker_RecZone.vars.zones[zn(Z["Razorfen Kraul"])].zones={zn(Z["The Barrens"])}
    Broker_RecZone.vars.zones[zn(Z["Scarlet Monastery"])].zones={zn(Z["Tirisfal Glades"])}
    Broker_RecZone.vars.zones[zn(Z["Razorfen Downs"])].zones={zn(Z["The Barrens"])}
    Broker_RecZone.vars.zones[zn(Z["Uldaman"])].zones={zn(Z["Badlands"])}
    Broker_RecZone.vars.zones[zn(Z["Maraudon"])].zones={zn(Z["Desolace"])}
    Broker_RecZone.vars.zones[zn(Z["Zul'Farrak"])].zones={zn(Z["Tanaris"])}
    Broker_RecZone.vars.zones[zn(Z["The Temple of Atal'Hakkar"])].zones={zn(Z["Swamp of Sorrows"])}
    Broker_RecZone.vars.zones[zn(Z["Blackrock Depths"])].zones={zn(Z["Blackrock Mountain"]),zn(Z["Molten Core"])}
    Broker_RecZone.vars.zones[zn(Z["Blackrock Spire"])].zones={zn(Z["Blackrock Mountain"]),zn(Z["Blackwing Lair"])}
    Broker_RecZone.vars.zones[zn(Z["Stratholme"])].zones={zn(Z["Eastern Plaguelands"])}
    Broker_RecZone.vars.zones[zn(Z["Dire Maul"])].zones={zn(Z["Feralas"])}
    Broker_RecZone.vars.zones[zn(Z["Scholomance"])].zones={zn(Z["Western Plaguelands"])}
    Broker_RecZone.vars.zones[zn(Z["Warsong Gulch"])].zones={zn(Z["The Barrens"])}
    Broker_RecZone.vars.zones[zn(Z["Warsong Gulch"])].zones={zn(Z["Ashenvale"])}
    Broker_RecZone.vars.zones[zn(Z["Arathi Basin"])].zones={zn(Z["Arathi Highlands"])}
    Broker_RecZone.vars.zones[zn(Z["Alterac Valley"])].zones={zn(Z["Alterac Mountains"])}
    Broker_RecZone.vars.zones[zn(Z["Blackwing Lair"])].zones={zn(Z["Blackrock Spire"])}
    Broker_RecZone.vars.zones[zn(Z["Molten Core"])].zones={zn(Z["Blackrock Depths"])}
    Broker_RecZone.vars.zones[zn(Z["Onyxia's Lair"])].zones={zn(Z["Dustwallow Marsh"])}
    Broker_RecZone.vars.zones[zn(Z["Zul'Gurub"])].zones={zn(Z["Stranglethorn Vale"])}
    Broker_RecZone.vars.zones[zn(Z["Ahn'Qiraj"])].zones={zn(Z["Silithus"])}
    Broker_RecZone.vars.zones[zn(Z["Temple of Ahn'Qiraj"])].zones={zn(Z["Silithus"])}
    Broker_RecZone.vars.zones[zn(Z["Zul'Aman"])].zones={zn(Z["Ghostlands"])}
    Broker_RecZone.vars.zones[zn(Z["Old Hillsbrad Foothills"])].zones={zn(Z["Tanaris"])}
    Broker_RecZone.vars.zones[zn(Z["The Black Morass"])].zones={zn(Z["Tanaris"])}
    Broker_RecZone.vars.zones[zn(Z["Hyjal Summit"])].zones={zn(Z["Tanaris"])}
    Broker_RecZone.vars.zones[zn(Z["The Culling of Stratholme"])].zones={zn(Z["Tanaris"])}

--["Hellfire Citadel"]
    Broker_RecZone.vars.zones[zn(Z["Hellfire Ramparts"])].zones={zn(Z["Hellfire Peninsula"])}
    Broker_RecZone.vars.zones[zn(Z["The Blood Furnace"])].zones={zn(Z["Hellfire Peninsula"])}
    Broker_RecZone.vars.zones[zn(Z["The Shattered Halls"])].zones={zn(Z["Hellfire Peninsula"])}
    Broker_RecZone.vars.zones[zn(Z["Magtheridon's Lair"])].zones={zn(Z["Hellfire Peninsula"])}
--["Tempest Keep"]
    Broker_RecZone.vars.zones[zn(Z["The Mechanar"])].zones={zn(Z["Netherstorm"])}
    Broker_RecZone.vars.zones[zn(Z["The Botanica"])].zones={zn(Z["Netherstorm"])}
    Broker_RecZone.vars.zones[zn(Z["The Arcatraz"])].zones={zn(Z["Netherstorm"])}
    Broker_RecZone.vars.zones[zn(Z["The Eye"])].zones={zn(Z["Netherstorm"])}    
--["Coilfang Reservoir"]
    Broker_RecZone.vars.zones[zn(Z["The Slave Pens"])].zones={zn(Z["Zangarmarsh"])}
    Broker_RecZone.vars.zones[zn(Z["The Underbog"])].zones={zn(Z["Zangarmarsh"])}
    Broker_RecZone.vars.zones[zn(Z["The Steamvault"])].zones={zn(Z["Zangarmarsh"])}
    Broker_RecZone.vars.zones[zn(Z["Serpentshrine Cavern"])].zones={zn(Z["Zangarmarsh"])}
--["Auchindoun"]
    Broker_RecZone.vars.zones[zn(Z["Auchenai Crypts"])].zones={zn(Z["Terokkar Forest"])}
    Broker_RecZone.vars.zones[zn(Z["Shadow Labyrinth"])].zones={zn(Z["Terokkar Forest"])}
    Broker_RecZone.vars.zones[zn(Z["Sethekk Halls"])].zones={zn(Z["Terokkar Forest"])}
    Broker_RecZone.vars.zones[zn(Z["Mana-Tombs"])].zones={zn(Z["Terokkar Forest"])}

    Broker_RecZone.vars.zones[zn(Z["Gruul's Lair"])].zones={zn(Z["Blade's Edge Mountains"])}
    Broker_RecZone.vars.zones[zn(Z["Black Temple"])].zones={zn(Z["Shadowmoon Valley"])}

--["Isle of Quel'Danas"]
    Broker_RecZone.vars.zones[zn(Z["Magisters' Terrace"])].zones={zn(Z["Isle of Quel'Danas"])}
    Broker_RecZone.vars.zones[zn(Z["Sunwell Plateau"])].zones={zn(Z["Isle of Quel'Danas"])}

    Broker_RecZone.vars.zones[zn(Z["Gadgetzan"])].zones={zn(Z["Tanaris"])}
    Broker_RecZone.vars.zones[zn(Z["Deeprun Tram"])].zones={zn(Z["Stormwind City"]),zn(Z["Ironforge"])}
    Broker_RecZone.vars.zones[zn(Z["Theramore Isle"])].zones={zn(Z["Dustwallow Marsh"]),znx(L["Boat"],1)}
    Broker_RecZone.vars.zones[zn(Z["Booty Bay"])].zones={zn(Z["Stranglethorn Vale"]),znx(L["Boat"],4)}
    Broker_RecZone.vars.zones[zn(Z["Ratchet"])].zones={zn(Z["The Barrens"]),znx(L["Boat"],4)}
    Broker_RecZone.vars.zones[zn(Z["Menethil Harbor"])].zones={zn(Z["Wetlands"]),znx(L["Boat"],1),znx(L["Boat"],7)}
    Broker_RecZone.vars.zones[zn(Z["Auberdine"])].zones={zn(Z["Darkshore"]),znx(L["Boat"],2),znx(L["Boat"],3)}
    Broker_RecZone.vars.zones[zn(Z["Grom'gol Base Camp"])].zones={zn(Z["Stranglethorn Vale"]),znx(L["Zeppelin"],2),znx(L["Zeppelin"],3)}
    Broker_RecZone.vars.zones[zn(Z["Blackrock Mountain"])].zones={zn(Z["Searing Gorge"]),zn(Z["Burning Steppes"]),zn(Z["Blackrock Depths"]),zn(Z["Blackrock Spire"])}

    Broker_RecZone.vars.zones[znx(L["Portal"],1)].zones={zn(Z["Shattrath City"]),zn(Z["Isle of Quel'Danas"])}
    Broker_RecZone.vars.zones[znx(L["Portal"],2)].zones={zn(Z["Shattrath City"]),zn(Z["Ironforge"])}
    Broker_RecZone.vars.zones[znx(L["Portal"],3)].zones={zn(Z["Shattrath City"]),zn(Z["Stormwind City"])}
    Broker_RecZone.vars.zones[znx(L["Portal"],4)].zones={zn(Z["Shattrath City"]),zn(Z["Darnassus"])}
    Broker_RecZone.vars.zones[znx(L["Portal"],5)].zones={zn(Z["Shattrath City"]),zn(Z["Undercity"])}
    Broker_RecZone.vars.zones[znx(L["Portal"],6)].zones={zn(Z["Shattrath City"]),zn(Z["Orgrimmar"])}
    Broker_RecZone.vars.zones[znx(L["Portal"],7)].zones={zn(Z["Shattrath City"]),zn(Z["Thunder Bluff"])}
    Broker_RecZone.vars.zones[znx(L["Portal"],8)].zones={zn(Z["Shattrath City"]),zn(Z["The Exodar"])}
    Broker_RecZone.vars.zones[znx(L["Portal"],9)].zones={zn(Z["Shattrath City"]),zn(Z["Silvermoon City"])}
    Broker_RecZone.vars.zones[znx(L["Portal"],10)].zones={zn(Z["Dalaran"]),zn(Z["Ironforge"])}
    Broker_RecZone.vars.zones[znx(L["Portal"],11)].zones={zn(Z["Dalaran"]),zn(Z["Stormwind City"])}
    Broker_RecZone.vars.zones[znx(L["Portal"],12)].zones={zn(Z["Dalaran"]),zn(Z["Darnassus"])}
    Broker_RecZone.vars.zones[znx(L["Portal"],13)].zones={zn(Z["Dalaran"]),zn(Z["The Exodar"])}
    Broker_RecZone.vars.zones[znx(L["Portal"],14)].zones={zn(Z["Dalaran"]),zn(Z["Shattrath City"])}
    Broker_RecZone.vars.zones[znx(L["Portal"],15)].zones={zn(Z["Dalaran"]),zn(Z["Undercity"])}
    Broker_RecZone.vars.zones[znx(L["Portal"],16)].zones={zn(Z["Dalaran"]),zn(Z["Orgrimmar"])}
    Broker_RecZone.vars.zones[znx(L["Portal"],17)].zones={zn(Z["Dalaran"]),zn(Z["Thunder Bluff"])}
    Broker_RecZone.vars.zones[znx(L["Portal"],18)].zones={zn(Z["Dalaran"]),zn(Z["Silvermoon City"])}
    Broker_RecZone.vars.zones[znx(L["Portal"],19)].zones={zn(Z["Dalaran"]),zn(Z["Shattrath City"])}

    Broker_RecZone.vars.zones[znx(L["Boat"],1)].zones={zn(Z["Theramore Isle"]),zn(Z["Menethil Harbor"])}
    Broker_RecZone.vars.zones[znx(L["Boat"],2)].zones={zn(Z["Stormwind City"]),zn(Z["Auberdine"])}
    Broker_RecZone.vars.zones[znx(L["Boat"],3)].zones={zn(Z["Darnassus"]),zn(Z["Auberdine"])}
    Broker_RecZone.vars.zones[znx(L["Boat"],4)].zones={zn(Z["Booty Bay"]),zn(Z["Ratchet"])}
    Broker_RecZone.vars.zones[znx(L["Boat"],5)].zones={zn(Z["Azuremyst Isle"]),zn(Z["Auberdine"])}
    Broker_RecZone.vars.zones[znx(L["Boat"],6)].zones={zn(Z["Stormwind City"]),zn(Z["Borean Tundra"])}
    Broker_RecZone.vars.zones[znx(L["Boat"],7)].zones={zn(Z["Menethil Harbor"]),zn(Z["Howling Fjord"])}
    Broker_RecZone.vars.zones[znx(L["Boat"],8)].zones={zn(Z["Borean Tundra"]),zn(Z["Dragonblight"])}
    Broker_RecZone.vars.zones[znx(L["Boat"],9)].zones={zn(Z["Howling Fjord"]),zn(Z["Dragonblight"])}
    Broker_RecZone.vars.zones[znx(L["Zeppelin"],1)].zones={zn(Z["Tirisfal Glades"]),zn(Z["Durotar"])}
    Broker_RecZone.vars.zones[znx(L["Zeppelin"],2)].zones={zn(Z["Tirisfal Glades"]),zn(Z["Grom'gol Base Camp"])}
    Broker_RecZone.vars.zones[znx(L["Zeppelin"],3)].zones={zn(Z["Durotar"]),zn(Z["Grom'gol Base Camp"])}
    Broker_RecZone.vars.zones[znx(L["Zeppelin"],4)].zones={zn(Z["Durotar"]),zn(Z["Borean Tundra"])}
    Broker_RecZone.vars.zones[znx(L["Zeppelin"],5)].zones={zn(Z["Tirisfal Glades"]),zn(Z["Howling Fjord"])}
    Broker_RecZone.vars.zones[zn(L["Orb of Translocation"])].zones={zn(Z["Silvermoon City"]),zn(Z["Undercity"])}
    Broker_RecZone.vars.zones[zn(Z["The Dark Portal"])].zones={zn(Z["Hellfire Peninsula"]),zn(Z["Blasted Lands"])}

--Northrend
    Broker_RecZone.vars.zones[zn(Z["Howling Fjord"])].zones={zn(Z["Grizzly Hills"]),zn(Z["Utgarde Keep"]),zn(Z["Utgarde Pinnacle"]),znx(L["Zeppelin"],6),znx(L["Boat"],9)}
    Broker_RecZone.vars.zones[zn(Z["Utgarde Keep"])].zones={zn(Z["Howling Fjord"])}
    Broker_RecZone.vars.zones[zn(Z["Utgarde Pinnacle"])].zones={zn(Z["Howling Fjord"])}
    Broker_RecZone.vars.zones[zn(Z["Borean Tundra"])].zones={zn(Z["Dragonblight"]),zn(Z["Sholazar Basin"]),zn(Z["Wintergrasp"]),zn(Z["The Nexus"]),zn(Z["The Oculus"]),zn(Z["The Eye of Eternity"]),znx(L["Boat"],8),znx(L["Zeppelin"],6)}
    Broker_RecZone.vars.zones[zn(Z["The Nexus"])].zones={zn(Z["Borean Tundra"])}
    Broker_RecZone.vars.zones[zn(Z["The Oculus"])].zones={zn(Z["Borean Tundra"])}
    Broker_RecZone.vars.zones[zn(Z["The Eye of Eternity"])].zones={zn(Z["Borean Tundra"])}
    Broker_RecZone.vars.zones[zn(Z["Dragonblight"])].zones={zn(Z["Borean Tundra"]),zn(Z["Dalaran"]),zn(Z["Zul'Drak"]),zn(Z["Grizzly Hills"]),zn(Z["Crystalsong Forest"]),zn(Z["Azjol-Nerub"]),zn(Z["Ahn'kahet: The Old Kingdom"]),zn(Z["The Obsidian Sanctum"]),zn(Z["Naxxramas"])}
    Broker_RecZone.vars.zones[zn(Z["Azjol-Nerub"])].zones={zn(Z["Dragonblight"])}
    Broker_RecZone.vars.zones[zn(Z["Ahn'kahet: The Old Kingdom"])].zones={zn(Z["Dragonblight"])}
    Broker_RecZone.vars.zones[zn(Z["The Obsidian Sanctum"])].zones={zn(Z["Dragonblight"])}
    Broker_RecZone.vars.zones[zn(Z["Naxxramas"])].zones={zn(Z["Dragonblight"])}
    Broker_RecZone.vars.zones[zn(Z["Grizzly Hills"])].zones={zn(Z["Howling Fjord"]),zn(Z["Zul'Drak"]),zn(Z["Dragonblight"]),zn(Z["Drak'Tharon Keep"])}
    Broker_RecZone.vars.zones[zn(Z["Drak'Tharon Keep"])].zones={zn(Z["Grizzly Hills"])}
    Broker_RecZone.vars.zones[zn(Z["Zul'Drak"])].zones={zn(Z["Grizzly Hills"]),zn(Z["Crystalsong Forest"]),zn(Z["Dragonblight"]),zn(Z["Gundrak"])}
    Broker_RecZone.vars.zones[zn(Z["Gundrak"])].zones={zn(Z["Zul'Drak"])}
    Broker_RecZone.vars.zones[zn(Z["Sholazar Basin"])].zones={zn(Z["Borean Tundra"]),zn(Z["Wintergrasp"])}
    Broker_RecZone.vars.zones[zn(Z["Crystalsong Forest"])].zones={zn(Z["Zul'Drak"]),zn(Z["Dragonblight"]),zn(Z["Icecrown"]),zn(Z["The Storm Peaks"])}
    Broker_RecZone.vars.zones[zn(Z["The Storm Peaks"])].zones={zn(Z["Crystalsong Forest"]),zn(Z["Halls of Stone"]),zn(Z["Halls of Lightning"])}
    Broker_RecZone.vars.zones[zn(Z["Halls of Stone"])].zones={zn(Z["The Storm Peaks"])}
    Broker_RecZone.vars.zones[zn(Z["Halls of Lightning"])].zones={zn(Z["The Storm Peaks"])}
    Broker_RecZone.vars.zones[zn(Z["Icecrown"])].zones={zn(Z["Crystalsong Forest"])}
    Broker_RecZone.vars.zones[zn(Z["Dalaran"])].zones={zn(Z["Dragonblight"]),zn(Z["The Violet Hold"]),znx(L["Portal"],10),znx(L["Portal"],11),znx(L["Portal"],12),znx(L["Portal"],13),znx(L["Portal"],14),znx(L["Portal"],15),znx(L["Portal"],16),znx(L["Portal"],17),znx(L["Portal"],18),znx(L["Portal"],19)}
    Broker_RecZone.vars.zones[zn(Z["The Violet Hold"])].zones={zn(Z["Dalaran"])}
    Broker_RecZone.vars.zones[zn(Z["Wintergrasp"])].zones={zn(Z["Borean Tundra"]),zn(Z["Dragonblight"]),zn(Z["Sholazar Basin"])}

end--do

-- Font Colors
local HEX_WHITE = "|cffffffff";
local HEX_RED = "|cffff0000";
local HEX_ORANGE = "|cffff7700";
local HEX_YELLOW = "|cffffff00";
local HEX_GREEN = "|cff00ff00";
local HEX_GRAY = "|cff777777";
local HEX_LIGHTBLUE = "|cff77ccff";


function Broker_RecZone:GetZoneColor(zone)
    -- PlayerLevel <= low for zone - 4  == RED
    -- PlayerLevel <  low for zone      == ORANGE
    -- PlayerLevel <= high              == YELLOW
    -- PlayerLevel <  high + 5          == GREEN
    -- Otherwise                        == GRAY
    if (zone.type == RZ_CITY and zone.faction == self.vars.playerFaction) then
        return HEX_WHITE;
    elseif (zone.type == RZ_SANCTUARY) then
        return HEX_LIGHTBLUE;
    elseif (self.vars.playerLevel <= (zone.low-4)) then
        return HEX_RED;
    elseif (self.vars.playerLevel < zone.low) then
        return HEX_ORANGE;
    elseif (self.vars.playerLevel <= zone.high) then
        return HEX_YELLOW;
    elseif (self.vars.playerLevel < (zone.high+5)) then
        return HEX_GREEN;
    else
        return HEX_GRAY;
    end
end

function Broker_RecZone:GetInstanceText(instance, colors)
    local color = HEX_WHITE;
    local text = "";
    if ( instance == nil ) then
        return "";
    end
    if (colors) then
        color = self:GetZoneColor(instance);
    end
    if (instance.type ~= RZ_INSTANCE) then
        text = instance.name..self.vars.types[instance.type];
    elseif (instance.low == instance.high) then
        text = instance.name..self.vars.types[instance.type]..
            " ("..instance.low.."+)";
    else
        text = instance.name.." ("..instance.low.."+ "..L["to "]..instance.high.."+)";
    end
    return color..text.."|r";
end

function Broker_RecZone:GetZoneText(zone, colors)
    local color = HEX_WHITE;
    if ( zone == nil ) then
        return "";
    end
    if (colors) then
        color = self:GetZoneColor(zone);
    end

    -- No level range for friendly city zones
    if (zone.type == RZ_CITY or zone.type == RZ_TRANSPORT) and zone.faction == self.vars.playerFaction then
        text = zone.name;
        color = HEX_WHITE;
    elseif (zone.faction == RZ_SANCTUARY) then
        text = zone.name;
        color = HEX_LIGHTBLUE;
    elseif (zone.type == RZ_INSTANCE) then
        if zone.low == zone.high then
            text = zone.name.." ("..zone.low.."+)" --  (60+)
        else
            text = zone.name.." ("..zone.low.."+ - "..zone.high.."+)" --  (50+ - 58+)
        end
    elseif (zone.type == RZ_BATTLEGROUND or zone.type == RZ_RAID20 or zone.type == RZ_RAID40 or zone.type == RZ_RAID25 or zone.type == RZ_RAID10 or zone.type == RZ_RAID10_20 or zone.type == RZ_PVP or zone.type == RZ_ARENA) then
        text = zone.name..self.vars.types[instance.type];
    else
        text = zone.name.." ("..zone.low.." - "..zone.high..")" -- (1 - 12)
    end
    -- Add zone name and level range.
    return color..text.."|r";
end

function Broker_RecZone:GetFactionColor(zone,colors)
    if not zone then 
        return "";
    end
    if colors then
        if ( zone.faction == RZ_CONTESTED ) then
            return HEX_YELLOW;
        elseif ( zone.faction == RZ_SANCTUARY ) then
            return HEX_LIGHTBLUE;
        elseif ( zone.faction == self.vars.playerFaction ) then
            return HEX_GREEN;
        else
            return HEX_RED;
        end
    end
    return "";
end

function Broker_RecZone:GetFactionText(zone,colors)
    local color = HEX_WHITE;
    if ( zone == nil ) then
        return "";
    end
    if (colors) then
        color = self:GetFactionColor(zone,colors);
    end
    return color..self.vars.factions[zone.faction].."|r";
end

function Broker_RecZone:GetContinentText(zone,colors)
    local color = HEX_WHITE;
    if ( zone == nil ) then
        return "";
    end
    if (colors) then
        if ( zone.continent == self.vars.current_continent ) then
            color = HEX_GREEN;
        else
            color = HEX_YELLOW;
        end
    end
    return color..self.vars.continents[zone.continent].."|r";
end

function Broker_RecZone:GetLevelText(zone,colors)
    local color = HEX_WHITE;
    if ( colors ) then
        color = self:GetZoneColor(zone);
    end
    if  zone.type == RZ_CITY and (zone.faction == self.vars.playerFaction or zone.faction == RZ_SANCTUARY) then
        return HEX_WHITE..self.vars.types[zone.type].."|r";
    elseif (zone.faction == RZ_SANCTUARY) then
        return HEX_LIGHTBLUE..self.vars.types[zone.type].."|r";
    elseif zone.type == RZ_INSTANCE then
        if zone.low == zone.high then
            return color..zone.low.."+|r"; -- 60+
        else            
            return color..zone.low.."+-"..zone.high.."+|r"; --  50+-58+
        end
    elseif zone.type == RZ_BATTLEGROUND or zone.type == RZ_RAID20 or zone.type == RZ_RAID40 or zone.type == RZ_RAID25 or zone.type == RZ_RAID10 or zone.type == RZ_RAID10_20 or zone.type == RZ_PVP or zone.type == RZ_ARENA then
        return color..self.vars.types[zone.type].."|r";
    end    
    if zone.low == zone.high then
        return color..tostring(zone.low).."|r";
    end
    return color..tostring(zone.low).."-"..tostring(zone.high).."|r";
end

function Broker_RecZone:GetPathText(zone,colors)
    local zonecolor = HEX_WHITE;
    local levelcolor = HEX_WHITE;
    if ( colors ) then
        zonecolor = self:GetFactionColor(zone,colors);
        levelcolor = self:GetZoneColor(zone);
    end
    if zone.type == RZ_CITY or zone.type == RZ_TRANSPORT then
        return zonecolor..zone.name.."|r";
    elseif zone.type == RZ_INSTANCE then
        if zone.low == zone.high then
            return zonecolor..zone.name.."|r "..levelcolor.."("..zone.low.."+)|r";
        else
            return zonecolor..zone.name.."|r "..levelcolor.."("..zone.low.."+-"..zone.high.."+)|r";
        end
    elseif zone.type == RZ_BATTLEGROUND or zone.type == RZ_RAID20 or zone.type == RZ_RAID40 or zone.type == RZ_RAID25 or zone.type == RZ_RAID10 or zone.type == RZ_RAID10_20 or zone.type == RZ_PVP or zone.type == RZ_ARENA then
        return zonecolor..zone.name.."|r"..levelcolor..self.vars.types[zone.type].."|r";
    end
    if zone.low == zone.high then
        return zonecolor..zone.name.."|r "..levelcolor.."("..zone.low..")|r";
    end
    return zonecolor..zone.name.."|r "..levelcolor.."("..zone.low.."-"..zone.high..")|r";
end

function Broker_RecZone:StartCoordsUpdate()
    local x, y = GetPlayerMapPosition( "player" );

    if( x == 0 and y == 0 ) then -- instance
        self:ScheduleRepeatingTimer( self.OnCoordUpdate, 60, self )
        self.vars.playerCoordX = 0;
        self.vars.playerCoordY = 0;
        self.vars.coordtext = '';
    else
        self:ScheduleRepeatingTimer( self.OnCoordUpdate, 0.1, self )
    end
end

function Broker_RecZone:StopCoordsUpdate()
    if self:IsEventScheduled("RecZone") then self:CancelScheduledEvent("RecZone") end
    self.vars.playerCoordX = 0;
    self.vars.playerCoordY = 0;
    self.vars.coordtext = '';
end

Broker_RecZone.vars.playerinworld = false;
function Broker_RecZone:OnInitialize()
    local AceConfigReg = LibStub( "AceConfigRegistry-3.0" );
    local AceConfigDialog = LibStub( "AceConfigDialog-3.0" );
    local defaults = {
        profile = {
            showCurInstance = true,
            showInstance = true,
            showBattleground = true,
            showRaid = false,
            showFaction = false,
            showContinent = false,
            showLoc = true,
            showMap = true,
            lower = 5,
            higher = 5,
            showColoredText = true,
            showZoneName = true,
            showSubZoneName = false,
            showCoords = true,
            showMapCoords = true,
            minimapBar = false,
        },
    };
    local options = {
        type = 'group',
        handler = self,
        args = {
            label = {
                type = 'group',
                name = L["Label"],
                desc = L["Label"],
                order = 1,
                args = {
                    zone = {
                        type = 'toggle',
                        name = L["Show Zone Name"],
                        desc = L["Show Zone Name"],
                        get = function( info )
                                  return info.handler.db.profile.showZoneName;
                              end,
                        set = function( info, value )
                                  info.handler.db.profile.showZoneName = value;
                                  info.handler:Update();
                              end,
                        order = 1,
                    },
                    subzone = {
                        type = 'toggle',
                        name = L["Show Sub Zone Name"],
                        desc = L["Show Sub Zone Name"],
                        get = function( info )
                                  return info.handler.db.profile.showSubZoneName;
                              end,
                        set = function( info, value )
                                  info.handler.db.profile.showSubZoneName = value;
                                  info.handler:Update();
                              end,
                        order = 2,
                    },
                    coords = {
                        type = 'toggle',
                        name = L["Show Coords"],
                        desc = L["Show Coords"],
                        get = function( info )
                                  return info.handler.db.profile.showCoords;
                              end,
                        set = function( info, value )
                                  info.handler.db.profile.showCoords = value;
                                  if( value ) then
                                      info.handler:StartCoordsUpdate();
                                  else
                                      info.handler:StopCoordsUpdate();
                                  end--if
                                  info.handler:Update();
                              end,
                        order = 3,
                    },
                },
            },
            tooltip = {
                type = 'group',
                name = L["Tooltip"],
                desc = L["Tooltip"],
                order = 2,
                args = {
                    curinstance = {
                        type = 'toggle',
                        name = L["Show Instances for current zone"],
                        desc = L["Show Instances for current zone"],
                        get = function( info )
                                  return info.handler.db.profile.showCurInstance;
                              end,
                        set = function( info, value )
                                  info.handler.db.profile.showCurInstance = value;
                                  info.handler:UpdateTooltip();
                              end,
                        order = 1,
                    },
                    faction = {
                        type = 'toggle',
                        name = L["Show Faction of zones"],
                        desc = L["Show Faction of zones"],
                        get = function( info )
                                  return info.handler.db.profile.showFaction;
                              end,
                        set = function( info, value )
                                  info.handler.db.profile.showFaction = value;
                                  info.handler:UpdateTooltip();
                              end,
                        order = 2,
                    },
                    continent = {
                        type = 'toggle',
                        name = L["Show Continent for zones"],
                        desc = L["Show Continent for zones"],
                        get = function( info )
                                  return info.handler.db.profile.showContinent;
                              end,
                        set = function( info, value )
                                  info.handler.db.profile.showContinent = value;
                                  info.handler:UpdateTooltip();
                              end,
                        order = 3,
                    },
                    instance = {
                        type = 'toggle',
                        name = L["Show Recommended Instances"],
                        desc = L["Show Recommended Instances"],
                        get = function( info )
                                  return info.handler.db.profile.showInstance;
                              end,
                        set = function( info, value )
                                  info.handler.db.profile.showInstance = value;
                                  info.handler:UpdateTooltip();
                              end,
                        order = 4,
                    },
                    battleground = {
                        type = 'toggle',
                        name = L["Show Battlegrounds"],
                        desc = L["Show Battlegrounds"],
                        get = function( info )
                                  return info.handler.db.profile.showBattleground;
                              end,
                        set = function( info, value )
                                  info.handler.db.profile.showBattleground = value;
                                  info.handler:UpdateTooltip();
                              end,
                        order = 5,
                    },
                    raid = {
                        type = 'toggle',
                        name = L["Show Raid Instances"],
                        desc = L["Show Raid Instances"],
                        get = function( info )
                                  return info.handler.db.profile.showRaid;
                              end,
                        set = function( info, value )
                                  info.handler.db.profile.showRaid = value;
                                  info.handler:UpdateTooltip();
                              end,
                        order = 6,
                    },
                    loc = {
                        type = 'toggle',
                        name = L["Show Location of Instances"],
                        desc = L["Show Location of Instances"],
                        get = function( info )
                                  return info.handler.db.profile.showLoc;
                              end,
                        set = function( info, value )
                                  info.handler.db.profile.showLoc = value;
                                  info.handler:UpdateTooltip();
                              end,
                        order = 7,
                    },
                    lower = {
                        type = 'range',
                        name = L["Recommend lower level zones"],
                        desc = L["Recommend lower level zones"],
                        max = 10;
                        min = 0;
                        step = 1;
                        get = function( info )
                                  return info.handler.db.profile.lower;
                              end,
                        set = function( info, value )
                                  info.handler.db.profile.lower = value;
                                  info.handler:UpdateTooltip();
                              end,
                        order = 8,
                    },
                    higher = {
                        type = 'range',
                        name = L["Recommend higher level zones"],
                        desc = L["Recommend higher level zones"],
                        max = 10;
                        min = 0;
                        step = 1;
                        get = function( info )
                                  return info.handler.db.profile.higher;
                              end,
                        set = function( info, value )
                                  info.handler.db.profile.higher = value;
                                  info.handler:UpdateTooltip();
                              end,
                        order = 9,
                    },
                },
            },
            map = {
                type = 'group',
                name = L["Map"],
                desc = L["Map"],
                order = 3,
                args = {
                    mapoverlay = {
                        type = 'toggle',
                        name = L["Show Zone Levels"],
                        desc = L["Show Zone Levels"],
                        get = function( info )
                                  return info.handler.db.profile.showMap;
                              end,
                        set = function( info, value )
                                  info.handler.db.profile.showMap = value;
                                  if( value ) then
                                      RecZoneMapText:Show();
                                  else
                                      RecZoneMapText:Hide();
                                  end
                              end,
                        order = 1,
                    },
                    mapcoords = {
                        type = 'toggle',
                        name = L["Show Coords"],
                        desc = L["Show Coords"],
                        get = function( info )
                                  return info.handler.db.profile.showMapCoords;
                              end,
                        set = function( info, value )
                                  info.handler.db.profile.showMapCoords = value;
                                  if( value ) then
                                      RecZoneMapFrameCursorCoords:Show();
                                      RecZoneMapFramePlayerCoords:Show();
                                  else
                                      RecZoneMapFrameCursorCoords:Hide();
                                      RecZoneMapFramePlayerCoords:Hide();
                                  end--if
                              end,
                        order = 2,
                    },
                },
            },
            minimapBar = {
                type = 'toggle',
                name = "Show minimap bar",
                desc = "Show the bar above the minimap that tells the location and allows you to close minimap",
                get = function( info )
                          return info.handler.db.profile.minimapBar
                      end,
                set = function( info, value )
                          info.handler.db.profile.minimapBar = value;
                          if( value ) then
                              MinimapBorderTop:Show();
                              MinimapToggleButton:Show();
                              MinimapZoneTextButton:Show();
                          else
                              MinimapBorderTop:Hide();
                              MinimapToggleButton:Hide();
                              MinimapZoneTextButton:Hide();
                          end--if
                          Jostle:Refresh()
                      end,
            },
            color = {
                type = 'toggle',
                name = L["Show Colored Text"],
                desc = L["Show Colored Text"],
                get = function( info )
                          return info.handler.db.profile.showColoredText;
                      end,
                set = function( info, value )
                          info.handler.db.profile.showColoredText = value;
                          info.handler:Update();
                      end,
                order = 91,
            },
        }
    };

    self.db = LibStub( "AceDB-3.0" ):New( "RecZoneDB", defaults );
    
    local profileOptions = LibStub( "AceDBOptions-3.0" ):GetOptionsTable( self.db );
    
    AceConfigReg:RegisterOptionsTable( "Broker_RecZone Main Options", options );
    AceConfigReg:RegisterOptionsTable( "Broker_RecZone Profile", profileOptions );
    AceConfigDialog:AddToBlizOptions( "Broker_RecZone Main Options", L["Broker_RecZone"] );
    AceConfigDialog:AddToBlizOptions( "Broker_RecZone Profile", L["Profile"], L["Broker_RecZone"] );

    if( LibStub( "LibDataBroker-1.1", true ) ) then
        self.ldbobj = LibStub( "LibDataBroker-1.1" ):NewDataObject( "Broker_RecZone", {
                type = "data source",
                icon = "Interface\\AddOns\\Broker_RecZone\\Icon",
                label = L["Recommended Zone"],
                text = "",
                OnClick = Broker_RecZone.OnClick;
                OnEnter = function( frame )
                              Broker_RecZone:RegisterTablet( frame );
                              Broker_RecZone:OnTooltipUpdate();
                          end,
                OnLeave = function( frame )
                              Tablet:Unregister( frame );
                          end,
            } );
        self:CreateTooltip();
    end--if

    self.vars.continents={ GetMapContinents() };
    self.vars.continents[255] = "Other";
    
    local mapFrame = CreateFrame("Frame", "RecZoneMapFrame", WorldMapFrame)
    mapFrame:SetScript("OnUpdate", self.MapFrame_OnUpdate)
    
    local text = mapFrame:CreateFontString("RecZoneMapText", "OVERLAY")
    local font, size = GameFontNormal:GetFont()
    text:SetFont(font, size, "OUTLINE")
    text:SetTextColor(GameFontNormal:GetTextColor())
    text:SetShadowColor(GameFontNormal:GetShadowColor())
    text:SetShadowOffset(GameFontNormal:GetShadowOffset())
    text:SetPoint("TOP", WorldMapFrameAreaLabel, "BOTTOM", 0, -5)

    local cursorCoords = mapFrame:CreateFontString("RecZoneMapFrameCursorCoords", "ARTWORK")
    cursorCoords:SetFont(GameFontNormal:GetFont())
    cursorCoords:SetTextColor(GameFontNormal:GetTextColor())
    cursorCoords:SetShadowColor(GameFontNormal:GetShadowColor())
    cursorCoords:SetShadowOffset(GameFontNormal:GetShadowOffset())
    cursorCoords:SetPoint("RIGHT", WorldMapFrame, "CENTER", -80, -367)
    
    local playerCoords = mapFrame:CreateFontString("RecZoneMapFramePlayerCoords", "ARTWORK")
    playerCoords:SetFont(GameFontNormal:GetFont())
    playerCoords:SetTextColor(GameFontNormal:GetTextColor())
    playerCoords:SetShadowColor(GameFontNormal:GetShadowColor())
    playerCoords:SetShadowOffset(GameFontNormal:GetShadowOffset())
    playerCoords:SetPoint("RIGHT", WorldMapFrame, "CENTER", 80, -367)
    
end

function Broker_RecZone:OnEnable()
    self:RegisterEvent("ZONE_CHANGED", "Update");
    self:RegisterEvent("ZONE_CHANGED_INDOORS", "Update");
    self:RegisterEvent("ZONE_CHANGED_NEW_AREA", "Update");
    self:RegisterEvent("MINIMAP_ZONE_CHANGED", "Update");
    self:RegisterEvent("PLAYER_ENTERING_WORLD", "OnEnteringWorld");
    self:RegisterEvent("PLAYER_LEVEL_UP", "Update");
    self.timerHandle = self:ScheduleRepeatingTimer( "Update", 2 );
    RecZoneMapFrame:Show();
    if( self.db.profile.showCoords ) then
        self:StartCoordsUpdate();
    else
        self:StopCoordsUpdate();
    end--if
    if( self.db.profile.showMap ) then
        RecZoneMapText:Show();
    else
        RecZoneMapText:Hide();
    end
    if( self.db.profile.showMapCoords ) then
        RecZoneMapFrameCursorCoords:Show();
        RecZoneMapFramePlayerCoords:Show();
    else
        RecZoneMapFrameCursorCoords:Hide();
        RecZoneMapFramePlayerCoords:Hide();
    end--if
    if( self.db.profile.minimapBar ) then
        MinimapBorderTop:Show();
        MinimapToggleButton:Show();
        MinimapZoneTextButton:Show();
    else
        MinimapBorderTop:Hide();
        MinimapToggleButton:Hide();
        MinimapZoneTextButton:Hide();
    end--if
    Jostle:Refresh()

    self.vars.playerinworld = true; -- Maybe not completly true, but good enough
end

function Broker_RecZone:OnDisable()
    Broker_RecZoneMapFrame:Hide();
    Broker_RecZoneMapFrameCursorCoords:Hide();
    Broker_RecZoneMapFramePlayerCoords:Hide();
    if( not self.db.profile.minimapBar ) then
        MinimapBorderTop:Hide();
        MinimapToggleButton:Hide();
        MinimapZoneTextButton:Hide();
        self.db.profile.minimapBar = not self.db.profile.minimapBar
    end--if
    self:CancelTimer( self.timerHandle );
end

function Broker_RecZone:OnProfileEnable()
    if self.vars.playerinworld then
        if( self.db.profile.showCoords ) then
            info.handler:StartCoordsUpdate();
        else
            info.handler:StopCoordsUpdate();
        end--if
        if( self.db.profile.showMap ) then
            RecZoneMapText:Show();
        else
            RecZoneMapText:Hide();
        end
        if( self.db.profile.showMapCoords ) then
            RecZoneMapFrameCursorCoords:Show();
            RecZoneMapFramePlayerCoords:Show();
        else
            RecZoneMapFrameCursorCoords:Hide();
            RecZoneMapFramePlayerCoords:Hide();
        end--if
        self:Update();
    end
end

function Broker_RecZone:OnEnteringWorld()
    if not self.vars.playerinworld then
        self.vars.playerinworld = true;
    end
    if self.db.profile.showCoords then self:StartCoordsUpdate() end
    self:Update();
end

function Broker_RecZone:OnCoordUpdate()
    local x, y = GetPlayerMapPosition("player")
    if self.db.profile.showCoords and x ~= 0 and y ~= 0 then
        if self.vars.playerCoordX ~= x or self.vars.playerCoordY ~= y then
            self.vars.coordtext = string.format('(%.0f, %.0f) ', x*100, y*100);
            self.vars.playerCoordX = x;
            self.vars.playerCoordY = y;
            self:OnTextUpdate();
        end
    else
        self.vars.playerCoordX = x;
        self.vars.playerCoordY = y;
        self.vars.coordtext = '';
        self:OnTextUpdate();
    end
end

function Broker_RecZone:OnDataUpdate()
    local i;
    self.vars.playerLevel = UnitLevel("player");
    self.vars.zoneName = GetRealZoneText();
    self.vars.subZoneName = GetSubZoneText();
    if ( select(2, UnitFactionGroup("player")) == FACTION_ALLIANCE ) then
        self.vars.playerFaction = RZ_ALLIANCE;
    else
        self.vars.playerFaction = RZ_HORDE;
    end

    self.vars.zonetext = "";
    if( self.db.profile.showZoneName ) then
        self.vars.zonetext = self.vars.zoneName.." ";
        if( self.db.profile.showSubZoneName ) then
            if ( self.vars.subZoneName ~= "" ) then
                self.vars.zonetext = self.vars.zoneName..": "..self.vars.subZoneName.." ";
            end
        end
    elseif( self.db.profile.showSubZoneName ) then
        if ( self.vars.subZoneName ~= "" ) then
            self.vars.zonetext = self.vars.subZoneName.." ";
        else
            self.vars.zonetext = self.vars.zoneName.." ";
        end
    end

    self.vars.leveltext = "";
    local found = false;        
    for i,zone in ipairs(self.vars.zones) do
        if (string.find(self.vars.zoneName, zone.name, 1, true)) then
            self.vars.current_continent = zone.continent;
            self.vars.current_zone = zone;
            self.vars.leveltext = self:GetLevelText(zone, self.db.profile.showColoredText);
            found = true;
            break
        end
    end

    if not found then
        if self.vars.zoneName and self.vars.zoneName ~= "" then
            self:Print('Unknown zone detected "%s"', self.vars.zoneName )
        end
        self.vars.current_continent = self.vars.zones[1].continent;
        self.vars.current_zone = self.vars.zones[1];
        self.vars.leveltext = self:GetLevelText(self.vars.zones[1], self.db.profile.showColoredText);
    end
    
    self.vars.zonetext = self:GetFactionColor(self.vars.current_zone,self.db.profile.showColoredText)..self.vars.zonetext;
end

function Broker_RecZone:Update()
    self:OnDataUpdate();
    self:OnTextUpdate();
end

function Broker_RecZone:OnTextUpdate()
    self.ldbobj.text = self.vars.zonetext .. self.vars.coordtext .. self.vars.leveltext;
end

function Broker_RecZone:OnTooltipUpdate()
    local c,i;
    local text = "";
    local lower = self.db.profile.lower;
    local higher = self.db.profile.higher;
    local colorTooltip = self.db.profile.showColoredText;

    local cat = Tablet:AddCategory('columns', 2,
                                                                 'child_textR', 1, 'child_textG', 1, 'child_textB', 0,
                                                                 'child_text2R', 1, 'child_text2G', 1,    'child_text2B', 1);

    if ( self.vars.current_zone.type >= RZ_INSTANCE ) then
        cat:AddLine("text", L["Current Zone:"], "text2", self:GetInstanceText(self.vars.current_zone, false));
        if ( self.vars.subZoneName ~= "" ) then
            cat:AddLine("text", L["Current Sub Zone:"], "text2", self.vars.subZoneName);
        end
    else
        cat:AddLine("text", L["Current Zone:"], "text2", self:GetZoneText(self.vars.current_zone, false));
        if ( self.vars.subZoneName ~= "" ) then
            cat:AddLine("text", L["Current Sub Zone:"], "text2", self.vars.subZoneName);
        end
        -- Show Coords
        if self.db.profile.showCoords then
            cat:AddLine("text", L["Coords:"], "text2", string.format('%.0f, %.0f', self.vars.playerCoordX*100, self.vars.playerCoordY*100));
        end
        -- Show faction
        if self.db.profile.showFaction then
            cat:AddLine("text", L["Faction:"], "text2", self:GetFactionText(self.vars.current_zone,colorTooltip));
        end
        -- show continent
        if self.db.profile.showContinent then
            cat:AddLine("text", L["Continent:"], "text2", self:GetContinentText(self.vars.current_zone,colorTooltip));
        end
    end

    -- Show border zones for current zone.
    if self.db.profile.showCurInstance then
        if ( self.vars.current_zone.zones ) then
          for _, i in ipairs(self.vars.current_zone.zones) do
              local f = self.vars.zones[i].faction;
              local t = self.vars.zones[i].type;
              if (t == RZ_ZONE or t == RZ_CITY) and (f == RZ_CONTESTED or f == self.vars.playerFaction or f == RZ_SANCTUARY) then
                    cat:AddLine("text", "Border Zone:", "text2", self:GetZoneText(self.vars.zones[i], colorTooltip));
                end
            end
        end
    end

    -- Show instances for current zone.
  if self.db.profile.showCurInstance then
        if ( self.vars.current_zone.zones ) then
          for _, i in ipairs(self.vars.current_zone.zones) do
              local f = self.vars.zones[i].faction;
              local t = self.vars.zones[i].type;
              if (t >= RZ_INSTANCE) and ((f == RZ_CONTESTED) or (f == self.vars.playerFaction)) then
                    cat:AddLine("text", L["Instance:"], "text2", self:GetInstanceText(self.vars.zones[i], colorTooltip));
                end
            end
        end
    end
    
    -- Show recommended zones.
    cat = Tablet:AddCategory("columns", 3, "text", L["Recommended Zones:"]);
    
    for i,zone in ipairs(self.vars.zones) do
        if zone.type == RZ_ZONE and
             (zone.low - higher) <= self.vars.playerLevel and 
            (zone.high + lower) >= self.vars.playerLevel and
            (zone.faction == RZ_CONTESTED or zone.faction == self.vars.playerFaction) then    

            -- Show faction
            local factionText;
            if ( self.db.profile.showFaction ) then
                factionText = self:GetFactionText(zone,colorTooltip);
            else
                factionText = "";
            end

            -- show continent
            local continentText;
            if ( self.db.profile.showContinent ) then
                continentText = self:GetContinentText(zone,colorTooltip);
            else
                continentText = "";
            end

            cat:AddLine("text", self:GetZoneText(zone,colorTooltip),
                        "text2", factionText,
                        "text3", continentText,
                        "func", "OnClickZone", "arg1", self, "arg2", zone);
        end
    end
    
    -- Show recommended instances/battlegrounds/raids
    if( self.db.profile.showInstance or self.db.profile.showRaid or self.db.profile.showBattleground ) then
        cat = Tablet:AddCategory("columns", 2, "text", L["Recommended Instances:"])
        for i, instance in ipairs(self.vars.zones) do
            if (instance.low - higher) <= self.vars.playerLevel and
                    (instance.high + lower) >= self.vars.playerLevel and
                    (instance.faction == RZ_CONTESTED or instance.faction == self.vars.playerFaction) and
                    ((instance.type == RZ_INSTANCE and self.db.profile.showInstance) or
                    ((instance.type == RZ_RAID20 or instance.type == RZ_RAID40 or instance.type == RZ_RAID25 or instance.type == RZ_RAID10) and self.db.profile.showRaid) or
                    (instance.type == RZ_BATTLEGROUND and self.db.profile.showBattleground)) then
                local text =  self:GetInstanceText(instance,colorTooltip);
                local locText = "";
                if ( self.db.profile.showLoc ) then
                    if ( instance.zones ) then
                        locText = "<"..self.vars.zones[instance.zones[1]].name..">";
                    end
                end
                cat:AddLine("text", text, "text2", locText,
                            "func", "OnClickZone", "arg1", self, "arg2", instance);
            end
        end
    end
    Tablet:SetHint(L["Hint"])
end

function Broker_RecZone:OnClick()
    if not self then
        self = RecZone;
    end
    if IsShiftKeyDown() then
        if ChatFrameEditBox:IsVisible() then
            local text = "";
            if( self.db.profile.showZoneName ) then
                text = self.vars.zoneName.." ";
                if( self.db.profile.showSubZoneName ) then
                    if ( self.vars.subZoneName ~= "" ) then
                        text = self.vars.zoneName..": "..self.vars.subZoneName.." ";
                    end
                end
            elseif( self.db.profile.showSubZoneName ) then
                if ( self.vars.subZoneName ~= "" ) then
                    text = self.vars.subZoneName.." ";
                else
                    text = self.vars.zoneName.." ";
                end
            end
            local x, y = GetPlayerMapPosition("player")
            text = text..string.format("%.0f, %.0f", x*100, y*100);
            ChatFrameEditBox:Insert(text);
        end
    end
end

function Broker_RecZone:OnClickZone(zone)
    local start = self:GetZoneNr(self.vars.current_zone);
    local goal = self:GetZoneNr(zone)
    local text = "";    

    self.vars.path = self:FindPath(start,goal);
    if not self.vars.path then
        self.vars.path = self:FindPath(start,goal,true); -- Try walking in opposing factions zones
    end
    self:CreateTooltip();
    if not self.vars.path then
        RecZoneTooltip:AddLine("No path to destination found!");
    else
        RecZoneTooltip:AddLine("Walk path");
        for n,i in ipairs(self.vars.path) do
            local text = self:GetPathText(self.vars.zones[i],self.db.profile.showColoredText);
            if n ~= table.getn(self.vars.path) then 
                RecZoneTooltip:AddLine(text.." ->");
            else
                RecZoneTooltip:AddLine(text);
            end
        end
    end
    RecZoneTooltip:Show();
end

function Broker_RecZone.OnTooltipShow( tooltip )
    tooltip.class:OnTooltipUpdate();
end

function Broker_RecZone:CreateTooltip()
    if( not self.tooltip ) then
        self.tooltip = CreateFrame( "GameTooltip", "Broker_RecZoneTooltip", Tablet20Frame, "GameTooltipTemplate" );
    end
    self.tooltip:SetOwner( Tablet20Frame, "ANCHOR_NONE" );
    self.tooltip:ClearAllPoints();
    if Tablet20Frame:GetLeft() < (GetScreenWidth() / 2) then
        self.tooltip:SetPoint( "LEFT","Tablet20Frame","RIGHT" );
    else
        self.tooltip:SetPoint( "RIGHT","Tablet20Frame","LEFT" );
    end
    self.tooltip.class = self;
    self.tooltip.OldOnShow = self.tooltip.GetScript( "OnShow" );
    self.tooltip.SetScript( "OnShow", self.OnTooltipShow );
    self.tooltip:Hide();
end

local function copytable(tbl,newitem)
    local newtbl = {};
    for _,a in ipairs(tbl) do
        table.insert(newtbl,a);
    end
    table.insert(newtbl,newitem);
    return newtbl;
end

function Broker_RecZone:FindPath(start,goal,opposing)
    local visited={};
    local path={start};
    local paths={path};
    visited[start]=true;
    while true do
        path = paths[1];
        local c = path[table.getn(path)];
        if self.vars.zones[c].zones then
            for _,d in ipairs(self.vars.zones[c].zones) do
                if not visited[d] then
                    local z = self.vars.zones[d];
                    if z.type == RZ_ZONE or z.faction == RZ_CONTESTED or z.faction == self.vars.playerFaction or opposing then
                        visited[d]=true;
                        local newpath = copytable(path,d);
                        if d == goal then
                            paths = nil;
                            visited = nil;
                            return newpath; -- Path found
                        end
                        table.insert(paths,newpath);
                    end
                end
            end
        end
        table.remove(paths,1);
        if table.getn(paths) == 0 then
            return; -- Nothing found
        end
    end
end

function Broker_RecZone:GetZoneNr(zone)
    for i,z in ipairs(self.vars.zones) do
        if string.find(z.name,zone.name,1,true) and z.faction==zone.faction then
            return i;
        end
    end
end


-- Worldmap function --

function Broker_RecZone:ToggleWorldMapText()
    if( self.db.profile.showMap ) then
        Broker_RecZoneMapFrame:Show();
    else
        Broker_RecZoneMapFrame:Hide();
    end
end

function Broker_RecZone.MapFrame_OnUpdate()
    local self = Broker_RecZone;
    local zoneText = "";
    if self:IsActive() then

        if( self.db.profile.showMapCoords ) then
            local OFFSET_X = 0.0022
            local OFFSET_Y = -0.0262

            local x, y = GetCursorPosition()
            x = x / WorldMapFrame:GetScale()
            y = y / WorldMapFrame:GetScale()
        
            local px, py = GetPlayerMapPosition("player")
            local centerX, centerY = WorldMapFrame:GetCenter()
            local width = WorldMapButton:GetWidth()
            local height = WorldMapButton:GetHeight()
            local adjustedX = (x - (centerX - (width/2))) / width
            local adjustedY = (centerY + (height/2) - y ) / height
            local cx = (adjustedX + OFFSET_X)
            local cy = (adjustedY + OFFSET_Y)
            
            Broker_RecZoneMapFrameCursorCoords:SetText(format("%s %.0f, %.0f", L["Cursor:"], 100 * cx, 100 * cy))
            Broker_RecZoneMapFramePlayerCoords:SetText(format("%s %.0f, %.0f", L["Player:"], 100 * px, 100 * py))
        end

        if( self.db.profile.showMap ) then
            if (WorldMapFrame.areaName ~= nil) then     
                for i, zone in ipairs(self.vars.zones) do
                    if (string.find(WorldMapFrame.areaName, zone.name, 1, true)) then
                        zoneText = format("%s(%d-%d)|r\n\n", self:GetZoneColor(zone), zone.low, zone.high);
                        if ( zone.zones ) then
                            local foundfirst = false;
                            for _, c  in ipairs(zone.zones) do
                                if self.vars.zones[c].type >= RZ_INSTANCE and
                                   (self.vars.zones[c].faction == RZ_CONTESTED or self.vars.zones[c].faction == self.vars.playerFaction) then
                                    if not foundfirst then
                                      zoneText = zoneText .. HEX_YELLOW..L["Instances:"].."|r\n";
                                        foundfirst = true;
                                    end
                                    zoneText = zoneText .. self:GetInstanceText(self.vars.zones[c], true) .. "\n";
                                end
                            end
                        end
                    end
                end
            end
            Broker_RecZoneMapText:SetText(zoneText);
        end
    end
end
